#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (c) 2024, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type
__author__ = ["A Mohamed Rafeek, Madhan Sankaranarayanan"]

import time
from ansible_collections.cisco.dnac.plugins.module_utils.dnac import (
    DnacBase
)


class NetworkProfileFunctions(DnacBase):
    """Class containing member attributes for network profile workflow manager module"""

    def __init__(self, module):
        super().__init__(module)

    def check_site_template(self, each_profile, profile_info):
        """
        Validate and retrieve site and template details for the given profile.

        Parameters:
            self (object): An instance of a class used for interacting with
                           Cisco Catalyst Center.
            each_profile (dict) - Playbook details containing network profile
            profile_info (dict) - Validate and add the site and template info
                to the existing profile information.

        Returns:
            self: Updated object with validated site and template information.
        """
        onboarding_templates = each_profile.get("onboarding_templates")
        day_n_templates = each_profile.get("day_n_templates")
        self.log("Check given template exist in the Catalyst Center for {0}, {1}".
                 format(onboarding_templates, day_n_templates), "INFO")

        self.log("Fetching onboarding template details for: {0}".format(onboarding_templates),
                 "DEBUG")
        if onboarding_templates and isinstance(onboarding_templates, list):
            profile_info["onboarding_templates"] = self.get_templates_details(
                onboarding_templates)

        self.log("Fetching day-N template details for: {0}".format(day_n_templates), "DEBUG")
        if day_n_templates and isinstance(day_n_templates, list):
            profile_info["day_n_templates"] = self.get_templates_details(day_n_templates)

        site_names = each_profile.get("site_names")
        if site_names:
            validated_sites = []
            for site in site_names:
                if site:
                    site_data = {}
                    site_data["site_exist"], site_data["site_id"] =\
                        self.get_site_id(site)
                    site_data["site_names"] = site
                    validated_sites.append(site_data)
                    self.log("Site '{0}' existence: {1}, Site ID: {2}".format(
                        site, site_data["site_exist"], site_data["site_id"]), "INFO")

                    child_sites = self.get_child_sites(site)
                    if child_sites:
                        self.log("Child sites found for '{0}': {1}".format(site, child_sites), "DEBUG")
                        validated_sites.extend(child_sites)

                    if not site_data["site_exist"]:
                        self.msg = 'Given site {0} not exist in Catalyst Center'.format(site)
                        self.log(self.msg, "ERROR")
                        self.set_operation_result("failed", False, self.msg,
                                                  "ERROR").check_return_status()

            if len(validated_sites) > 0:
                # Filter duplicate site ids from site response
                unique_sites = [dict(final_site)
                                for final_site in {frozenset(site.items())
                                                   for site in validated_sites}]
                self.log("Found Site ID(s) list: {0}".format(self.pprint(unique_sites)), "INFO")
                profile_info["site_response"] = unique_sites

            return self

    def get_child_sites(self, site_name_hierarchy):
        """
        Retrieve child sites mapped to the given site hierarchy.

        Parameters:
            self (object): An instance of a class used for interacting with
                           Cisco Catalyst Center.
            site_name_hierarchy (str) - Site name with complete hierarchy

        Returns:
            list or None: List of child site details if found, otherwise None.
        """
        get_sites_params = {"name_hierarchy": site_name_hierarchy + "/.*"}
        self.log("Fetching child sites for '{}'. Request parameters: {}".format(
            site_name_hierarchy, get_sites_params), "DEBUG")

        try:
            response = self.execute_get_request("site_design", "get_sites", get_sites_params)
            self.log("Response from get_sites request: {}".format(response), "DEBUG")

            if response and isinstance(response, dict):
                child_sites = response.get("response", [])
                self.log("Found {0} child sites for site area: '{1}'".format(
                    len(child_sites), site_name_hierarchy), "DEBUG")

                if not child_sites:
                    self.log("No child sites found for '{0}'. Returning None.".format(
                        site_name_hierarchy), "DEBUG")
                    return None

                child_site_response = []
                for child in child_sites:
                    child_site_id = child.get("id")
                    child_site_name_hierarchy = child.get("nameHierarchy")
                    self.log("Received child site: '{0}' with ID: '{1}'".
                             format(child_site_name_hierarchy, child_site_id), "DEBUG")

                    if child_site_id and child_site_name_hierarchy:
                        self.log("Processing child site: '{0}' (ID: '{1}')".format(
                            child_site_name_hierarchy, child_site_id), "DEBUG")
                        child_site_response.append({
                            "site_exist": True,
                            "site_id": child_site_id,
                            "site_names": child_site_name_hierarchy
                        })

                self.log("All child sites for site area: '{0}': {1}".format(
                    site_name_hierarchy, self.pprint(child_site_response)), "DEBUG")
                return child_site_response

        except Exception as e:
            self.msg = 'Error retrieving child site(s): No child site(s) found for {0}. '.format(
                site_name_hierarchy)
            self.log(self.msg + str(e), "ERROR")
            self.set_operation_result("failed", False, self.msg, "ERROR")
            return None

    def get_templates_details(self, template_list):
        """
        Retrieve and validate the given list of templates from Cisco Catalyst Center.

        Parameters:
            self (object): An instance of a class used for interacting with Cisco Catalyst Center.
            template_list (list): List of template names to validate.

        Returns:
            list or None: List of dictionaries containing template details if found, otherwise None.

        Description:
            This function used to get the templates details from Cisco Catalyst Center and
            compare with given template_list from play book and retrun with list of
            template details.
        """
        self.log("Fetching details for templates: {}".format(template_list), "INFO")

        try:
            response = self.dnac._exec(
                family="configuration_templates",
                function="gets_the_templates_available",
                params={}
            )
            self.log("Response from gets_the_templates_available API: {0}".
                     format(self.pprint(response)), "DEBUG")

            if not response or not isinstance(response, list):
                self.log("Invalid or empty response received. No templates found.", "WARNING")
                return None

            template_info_list = []
            for input_template in template_list:
                template_info = {}
                for each_template in response:
                    if each_template.get("name") == input_template:
                        template_info = {
                            "template_name": each_template.get("name"),
                            "template_exist": True,
                            "template_id": each_template.get("templateId"),
                            "template_info": each_template
                        }
                        template_info_list.append(template_info)
                        self.log("Template '{0}' found with ID '{1}'.".format(
                            input_template, each_template.get("templateId")), "DEBUG")
                        break

                if not template_info:
                    self.msg = "Given template '{0}' does not exist in Catalyst Center".format(
                        input_template)
                    self.log(self.msg, "ERROR")
                    self.fail_and_exit(self.msg)

            if len(template_info_list) > 0:
                self.log("Final validated template details: {0}".format(
                    self.pprint(template_info_list)), "INFO")
                return template_info_list
            else:
                self.log("No valid templates found in the given list.", "WARNING")
                return None

        except Exception as e:
            self.msg = 'Error on retrieving templates: No template list received'
            self.log(self.msg + str(e), "ERROR")
            self.set_operation_result("failed", False, self.msg, "ERROR")
            return None

    def get_network_profile(self, profile_type, offset, limit):
        """
        Get network profile list from Cisco Catalyst Center based on the profile type
        given in the playbook and response with network profile information.

        Parameters:
            self (object): An instance of a class used for interacting with Cisco Catalyst Center.
            profile_type (str): A string containing Wireless, Switch or Assurance.
            offset (int): Offset value start from 1
            limit (int): Limit value start from 1 to 500

        Returns:
            list: A list of dict contains network profile information.

        Description:
            This function used to get the network profile based on input config.
        """

        param = {"type": profile_type,
                 "offset": offset,
                 "limit": limit}
        self.log("Fetching network profiles for profile type: {0}".format(profile_type), "INFO")

        try:
            response = self.execute_get_request(
                "site_design", "retrieves_the_list_of_network_profiles_for_sites", param)
            self.log("Response from retrieves_the_list_of_network_profiles_for_sites API: {0}".
                     format(self.pprint(response)), "DEBUG")

            profiles = response.get("response")
            if not profiles or not isinstance(profiles, list):
                self.log("Invalid or missing network profile response, expected dict but got {0}".
                         format(type(profiles).__name__), "ERROR")
                return None

            self.log("Received network profile response: {0}".format(self.pprint(profiles)), "INFO")
            return profiles

        except Exception as e:
            self.msg = "Error on retrieving {0} profile list: Unable to get the profile list. ".format(
                profile_type)
            self.log(self.msg + str(e), "ERROR")
            return None

    def get_templates_for_profile(self, profile_id):
        """
        Get the CLI template for the specific profile id
        Parameters:
            self (object): An instance of a class used for interacting with Cisco Catalyst Center.
            profile_id (str): A string containing profile id to fetch template assigned to
                the profile.

        Returns:
            list: A list of dict contains network template ids and names.

        Description:
            This function is used to get cli template for the specific profile id
        """
        self.log("Fetching CLI templates for profile ID: {0}".format(profile_id), "INFO")
        param = {
            "profile_id": profile_id
        }

        try:
            response = self.execute_get_request(
                "network_settings", "retrieve_cli_templates_attached_to_a_network_profile",
                param)
            self.log("Response from retrieve_cli_templates_attached_to_a_network_profile " +
                     "API: {0}".format(self.pprint(response)), "DEBUG")

            templates = response.get("response")
            if not templates or not isinstance(templates, list):
                self.log("Invalid or missing template response, expected list but got {0}".
                         format(type(templates).__name__), "ERROR")
                return None

            self.log("CLI templates retrieved: {0}".format(self.pprint(templates)), "INFO")
            return templates

        except Exception as e:
            self.msg = "Error on retrieving templates for profile: Unable to retrieve the templates " +\
                "for profile '{0}'".format(profile_id)
            self.log(self.msg + str(e), "ERROR")
            return None

    def attach_networkprofile_cli_template(self, profile_name, profile_id, template_name,
                                           template_id):
        """
        Attaches a network profile to a CLI template using the given profile and template details.

        Parameters:
            self (object): An instance of a class used for interacting with Cisco Catalyst Center.
            profile_name (str): Name of the network profile.
            profile_id (str): Unique identifier of the network profile.
            template_name (str): Name of the CLI template.
            template_id (str): Unique identifier of the CLI template.

        Returns:
            dict: Contains network profile information if successful.
            None: If the operation fails.

        Description:
            Attaches a given network profile to a CLI template by sending the necessary
            request payload.
        """
        self.log("Attaching CLI template '{0}' (ID: {1}) to profile '{2}' (ID: {3})".format(
            template_name, template_id, profile_name, profile_id), "INFO")
        function_name = "attach_network_profile_to_a_day_n_cli_template"
        profile_payload = {
            "profileId": profile_id,
            "template_id": template_id
        }

        try:
            return self.execute_process_task_data("configuration_templates",
                                                  function_name, profile_payload)

        except Exception as e:
            error_msg = "Error attaching template(s): Unable to attach profile '{0}' to CLI template '{1}'.".format(
                profile_name, template_name)
            self.log(error_msg + str(e), "ERROR")
            self.set_operation_result("failed", False, error_msg, "ERROR")
            return None

    def detach_networkprofile_cli_template(self, profile_name, profile_id, template_name,
                                           template_id):
        """
        Detaches a network profile from a CLI template using the provided profile and template IDs.

        Parameters:
            profile_name (str): Name of the network profile.
            profile_id (str): Unique identifier of the network profile.
            template_name (str): Name of the CLI template.
            template_id (str): Unique identifier of the CLI template.

        Returns:
            dict: Contains network profile information if successful.
            None: If the operation fails.

        Description:
            Detaches the specified network profile from a CLI template by sending
            the necessary request payload.
        """

        self.log("Detaching CLI template '{0}' (ID: {1}) from network profile '{2}' (ID: {3})".
                 format(template_name, template_id, profile_name, profile_id), "INFO")
        function_name = "detach_a_list_of_network_profiles_from_a_day_n_cli_template"
        profile_payload = {
            "profile_id": profile_id,
            "template_id": template_id
        }
        try:
            return self.execute_process_task_data("configuration_templates",
                                                  function_name, profile_payload)

        except Exception as e:
            error_msg = "Error on detach template(s): Unable to detach network profile '{0}' from CLI template(s) '{1}'. ".format(
                profile_name, template_name)
            self.log(error_msg + str(e), "ERROR")
            self.set_operation_result("failed", False, error_msg, "ERROR")
            return None

    def process_templates(self, templates, previous_templates, profile_name, profile_id):
        """
        Checks and assigns templates from the input config to a network profile.

        Parameters:
            self (object): An instance of a class used for interacting with Cisco Catalyst Center.
            templates (list): A list of dictionaries, each containing template details from input config.
            previous_templates (list): A list of dictionaries with template name and ID already assigned to the profile.
            profile_name (str): A string containing profile name used to assign template to profile.
            profile_id (str): A string containing profile id used to assign the onboarding or
                              day n template.

        Returns:
            list: A list contains templates assigned to the profile status.
        """
        self.log("Processing {0} templates for profile: {1}".format(len(templates),
                                                                    profile_name), "DEBUG")
        template_response = []

        for each_template in templates:
            template_name = each_template.get("template_name")
            self.log("Checking template: {0}".format(template_name), "DEBUG")

            template_exist = each_template.get("template_exist")
            if not template_exist:
                self.log("Template '{0}' does not exist, skipping.".format(template_name), "DEBUG")
                continue  # Skip the rest of the loop if template doesn't exist

            template_id = each_template.get("template_id")
            self.log("Template '{0}' exists, attaching network profile.".format(
                template_name), "DEBUG")

            # If no previous templates, we can directly attach
            if not previous_templates:
                self.log("No previous templates to check, attaching '{0}'.".format(
                    template_name), "DEBUG")

                template_status = self.attach_networkprofile_cli_template(
                    profile_name, profile_id, template_name, template_id)
                if template_status.get("progress"):
                    msg = "Template '{0}' successfully attached to the network profile".format(
                        template_name
                    )
                    template_response.append(msg)

                continue  # Continue to the next template

            # If template already exists in previous templates, skip it
            if self.value_exists(previous_templates, "name", template_name):
                self.log("Template '{0}' already exists in previous templates, skipping.".
                         format(template_name), "DEBUG")
                continue  # Skip the rest of the loop if template already exists in previous_templates

            # Otherwise, attach the template
            self.log("Template '{0}' not found in previous templates, attaching..".format(
                template_name), "DEBUG")
            template_status = self.attach_networkprofile_cli_template(
                profile_name, profile_id, template_name, template_id)

            if template_status.get("progress"):
                msg = "Template '{0}' successfully attached to the network profile".format(
                    template_name
                )
                template_response.append(msg)

        self.log("Finished processing templates. Total attached: {0}".format(
            len(template_response)), "DEBUG")
        return template_response

    def get_site_lists_for_profile(self, profile_name, profile_id):
        """
        Retrieves the list of site IDs assigned to a specific network profile.

        Parameters:
            self (object): An instance of a class used for interacting with Cisco Catalyst Center.
            profile_name (str): Name of the network profile.
            profile_id (str): A string containing profile id to fetch Sites assigned to
                the profile.

        Returns:
            list: A list of dictionaries containing site IDs assigned to the profile,
            or None if the request fails.

        Description:
            This function is used to get site id list for the specific profile id
        """
        self.log("Fetching site list for profile {0} ID: {1}".format(profile_name,
                                                                     profile_id), "INFO")
        param = {
            "profile_id": profile_id
        }
        func_name = "retrieves_the_list_of_sites_that_the_given_network_profile_for_sites_is_assigned_to"

        try:
            response = self.execute_get_request("site_design", func_name, param)
            self.log("Response from get site lists for profile API: {0}".
                     format(self.pprint(response)), "DEBUG")

            if not response:
                self.log("Invalid or missing Site list response, expected list but got {0}".
                         format(type(response).__name__), "ERROR")
                return None

            site_list = response.get("response")
            self.log("Retrieved site list: {0}".format(self.pprint(site_list)), "DEBUG")
            return site_list

        except Exception as e:
            msg = "Error retrieving site list for profile '{0}': {1}".format(
                profile_name, profile_id)
            self.log(msg + str(e), "ERROR")
            self.set_operation_result("failed", False, msg, "INFO")
            return None

    def compare_config_with_sites_templates(self, each_config, data_list, config_type):
        """
        Compare input profile config templates or sites with existing assigned profile data.

        Parameters:
            self (object): An instance of a class used for interacting with Cisco Catalyst Center.
            each_config (dict): A dict containing profile name, template name to compare existing
                                template assigned profile template.
            data_list (list): List of dict contains template id and name assigned to the profile.

        Returns:
            tuple: (status (bool), unmatched items list or None)

        Description:
            This function is used to compare the template names and return status and unmatched
            template names as response.
        """
        if config_type == "template":
            un_match_template = []
            matched_template = []
            try:
                for template_type in ["onboarding_templates", "day_n_templates"]:
                    tempaltes = each_config.get(template_type)
                    if tempaltes:
                        for template in tempaltes:
                            if not self.value_exists(data_list, "name", template):
                                self.log("Found Unmatched template: {0}".format(template), "DEBUG")
                                un_match_template.append(template)
                            else:
                                self.log("Matched template: {0}".format(template), "DEBUG")
                                matched_template.append(template)

                if matched_template and data_list and\
                   len(matched_template) <= len(data_list) and not un_match_template:
                    self.log("Given templates: {0} are matched with existing template: {1}".
                             format(data_list, each_config.get(template_type)), "DEBUG")
                    return True, matched_template

                if not matched_template and not each_config.get("onboarding_templates") and not each_config.get("day_n_templates"):
                    return True, matched_template

                return False, un_match_template

            except Exception as e:
                msg = "Error comparing template: Unable to compare config '{0}' with existing config '{1}'".format(
                    each_config, data_list)
                self.log(msg + str(e), "ERROR")
                self.fail_and_exit(msg)

        elif config_type == "sites":
            un_match_site_ids = []
            matched_site_ids = []
            try:
                if each_config:
                    for site in each_config:
                        self.log("Checking site: {0} in existing data: {1}.".format(
                            self.pprint(site), self.pprint(data_list)), "DEBUG")
                        if not self.value_exists(data_list, "id", site["site_id"]):
                            un_match_site_ids.append(site["site_names"])
                        else:
                            matched_site_ids.append(site["site_names"])

                if un_match_site_ids:
                    self.log("Found Unmatched site IDs: {0}.".format(
                        self.pprint(un_match_site_ids)), "DEBUG")
                    return False, un_match_site_ids

                if len(matched_site_ids) <= len(data_list) and not un_match_site_ids:
                    self.log("Site IDs are matched: {0}.".format(
                        self.pprint(matched_site_ids)), "DEBUG")
                    return True, None
                else:
                    return True, None

            except Exception as e:
                msg = "Error on site name comparison: Unable to compare config {0} with existing {1}".format(
                    each_config, data_list)
                self.log(msg + str(e), "ERROR")
                self.fail_and_exit(msg)

        else:
            msg = "compare_config_with_sites_templates: Invalid config_type. Expected 'template' or 'sites', got '{0}'.".format(config_type)
            self.fail_and_exit(msg)

    def assign_site_to_network_profile(self, profile_id, site_id, profile_name, site_name):
        """
        Assign a site to a network profile.

        Parameters:
            self (object): An instance of a class used for interacting with Cisco Catalyst Center.
            profile_id (str): A string containing profile id used to assign to site.
            site_id (str): A string containing site id used to assign the site id to profile.
            profile_name (str): A string containing profile name used to assign site to profile.
            site_name (str): A string containing site name used to assign the site to profile.

        Returns:
            dict: A dict contains Task details of the profile assigned status.

        Description:
            This function is used to assign the Profile id to the Site.
        """
        self.log("Assigning site {0}: {1} to profile {2}:{3}.".format(
            site_name, site_id, profile_name, profile_id), "INFO")
        params = {
            "profile_id": profile_id,
            "id": site_id
        }

        try:
            return self.execute_process_task_data(
                "site_design", "assign_a_network_profile_for_sites_to_the_given_site",
                params
            )
        except Exception as e:
            error_msg = "Failed to assign the site(s) '{0}' to the profile '{1}'".format(
                site_name, profile_name)
            self.log(error_msg + str(e), "ERROR")
            self.set_operation_result("failed", False, error_msg, "ERROR")
            return None

    def unassign_site_to_network_profile(self, profile_name, profile_id, site_name, site_id):
        """
        Un assign a site from the network profile.

        Parameters:
            self (object): An instance of a class used for interacting with Cisco Catalyst Center.
            profile_name (str): A string containing profile name used to unassign from site.
            profile_id (str): A string containing profile id used to unassign from site.
            site_name (str): A string containing site name used to un assign from the profile.
            site_id (str): A string containing site id used to un assign from the profile.

        Returns:
            dict: A dict contains Task details of the profile unassigned status.

        Description:
            This function is used to un assign the Profile id from the Site.
        """
        param = {
            "profile_id": profile_id,
            "site_id": site_id
        }
        self.log("Unassigning site {0}: {1} from network profile {2}: {3}.".
                 format(site_name, site_id, profile_name, profile_id), "INFO")

        try:
            return self.execute_process_task_data(
                "site_design", "unassigns_a_network_profile_for_sites_from_multiple_sites",
                param
            )
        except Exception as e:
            error_msg = "Failed to unassign site(s) '{0}' from profile '{1}'. ".format(
                site_name, profile_name)
            self.log(error_msg + str(e), "ERROR")
            self.set_operation_result("failed", False, error_msg, "ERROR")
            return None

    def delete_network_profiles(self, profile_name, profile_id):
        """
        Delete network profiles from the Catalyst Center and response with
        the task details.

        Parameters:
            self (object): An instance of a class used for interacting with Cisco Catalyst Center.
            profile_name (str): Contains profile_name to be deleted.
            profile_id (str): Contains profile ID to be deleted.

        Returns:
            dict: A dict contains task details of the deleted status.

        Description:
            This function is used to delete network profiles from Catalyst Center.
        """

        self.log("Delete network profiles for {0}: {1}".format(profile_name, profile_id), "INFO")
        param = {
            "id": profile_id,
        }

        try:
            return self.execute_process_task_data(
                "site_design", "deletes_a_network_profile_for_sites", param
            )
        except Exception as e:
            error_msg = "Failed to delete network profile '{0}'. ".format(profile_name)
            self.log(error_msg + str(e), "ERROR")
            self.set_operation_result("failed", False, error_msg, "ERROR")
            return None

    def execute_process_task_data(self, profile_family, profile_function_name,
                                  payload_data, task_id=None):
        """
        This function used to execute the payload data based on the family and function
        and get the task id, aslo pass the taskid and get details of the task as a output.
        Parameters:
            self (object): An instance of a class used for interacting with Cisco Catalyst Center.
            profile_family (str): A string containing family name of the API.
            profile_function_name (str): A string containing function name of the API.
            payload_data (dict): Contains dict of payload for the API.
            task_id (str): Optional param if task id is available profile_family,
                        profile_function_name, payload_data can be dummy data
        Returns:
            dict: A dict contains Task details as output.

        Description:
            This function is used to process the API call and show the task details
            as the response.
        """
        try:
            if not profile_family or not profile_function_name or not payload_data:
                self.log("Invalid API parameters provided.", "ERROR")
                return None

            if not task_id:
                task_id = self.get_taskid_post_api_call(profile_family,
                                                        profile_function_name,
                                                        payload_data)
                if not task_id:
                    self.log("Failed to retrieve task ID.", "ERROR")
                    return None

            if not isinstance(self.payload, dict):
                self.log("self.payload is not a valid dictionary.", "ERROR")
                return None

            resync_retry_count = int(self.payload.get("dnac_api_task_timeout", 10))
            resync_retry_interval = int(self.payload.get("dnac_task_poll_interval", 5))
            while resync_retry_count > 0:
                task_details_response = self.get_tasks_by_id(task_id)

                if not task_details_response:  # Ensure the response is valid
                    self.log("Failed to retrieve task details for task ID: {0}".
                             format(task_id), "ERROR")
                    return None

                task_status = task_details_response.get("status")
                self.log("Task ID: {0}, Status: {1}, Attempts remaining: {2}".format(
                    task_id, task_status, resync_retry_count), "INFO")

                if task_details_response.get("endTime") is not None:
                    if task_status == "SUCCESS":
                        self.result['changed'] = True
                        self.result['response'] = self.get_task_details_by_id(task_id)
                        return self.result['response']

                    if task_status == "FAILURE":
                        self.result['changed'] = False
                        self.result['response'] = self.get_task_details_by_id(task_id)
                        return self.result['response']

                self.log("Pauses execution for {0} seconds.".format(resync_retry_interval), "INFO")
                time.sleep(resync_retry_interval)
                resync_retry_count = resync_retry_count - 1

            self.log("Task {0} did not complete within the timeout.".format(task_id), "ERROR")
            return None

        except Exception as e:
            self.msg = 'An error occurred during get task details. '
            self.log(self.msg + str(e), "ERROR")
            self.fail_and_exit(self.msg)

    def value_exists(self, data, target_key, target_value):
        """
        Check if a specific key-value pair exists in a dictionary or a list of dictionaries.

        Parameters:
            self (object): An instance of a class used for interacting with Cisco Catalyst Center.
            data (dict | list): The dictionary or list of dictionaries to search in.
            target_key (str): A string containing key of the dict in the data.
            target_value (str): A string containing value to find in the data.

        Returns:
            bool: True if the key-value pair exists, otherwise False.

        Description:
            This function recursively searches for the key-value pair in nested
            dictionaries and lists.
        """
        if isinstance(data, dict):
            # First, check if the key exists in the dictionary at the top level
            if data.get(target_key) == target_value:
                return True

            # Then, recursively check values (nested dictionaries/lists)
            for value in data.values():
                if self.value_exists(value, target_key, target_value):
                    return True
        elif isinstance(data, list):
            for item in data:
                if self.value_exists(item, target_key, target_value):
                    return True
        return False

    def find_duplicate_value(self, config_list, key_name):
        """
        Identifies duplicate values for a given key in a list of dictionaries.

        Parameters:
            config_list (list of dict): A list where each dictionary contains key-value pairs.
            key_name (str): The key whose values need to be checked for duplicates.

        Returns:
            list: A list of duplicate key_name values found in the input list.
        """
        seen = set()
        duplicates = set()

        for item in config_list:  # Ensure the item is a dictionary
            value = item.get(key_name)
            if value:
                if value in seen:
                    duplicates.add(value)
                else:
                    seen.add(value)

        return list(duplicates)

    def deduplicate_list_of_dict(self, list_of_dicts):
        """
        Removes duplicate dictionaries from a list.

        Args:
            list_of_dicts (list): A list of dictionaries to deduplicate.

        Returns:
            list: A list of unique dictionaries (duplicates removed).

        Description:
            Iterates through a list of dictionaries and removes duplicates based on their content.
            Uses a content-based comparison approach where dictionaries with identical key-value pairs
            are considered duplicates. The first occurrence of each unique dictionary is preserved.
            Empty lists and non-list inputs are handled gracefully.
        """
        self.log("Initiating deduplication process for dictionary list", "DEBUG")

        # Input validation
        if not isinstance(list_of_dicts, list):
            self.log("Input is not a list, returning empty list. Input type: {0}".format(
                type(list_of_dicts).__name__), "WARNING")
            return []

        if not list_of_dicts:
            self.log("Empty list provided for deduplication - returning empty list", "DEBUG")
            return []

        original_count = len(list_of_dicts)
        self.log("Starting deduplication for list with {0} dictionaries: {1}".format(
            original_count, self.pprint(list_of_dicts)), "DEBUG")

        seen = set()
        unique_dicts = []
        duplicates_found = 0

        for index, d in enumerate(list_of_dicts):
            # Validate that each item is a dictionary
            if not isinstance(d, dict):
                self.log("Skipping non-dictionary item at index {0}: {1} (type: {2})".format(
                    index, d, type(d).__name__), "WARNING")
                continue

            # Convert dictionary to a tuple of sorted items (hashable representation)
            try:
                identifier = tuple(sorted(d.items()))
            except TypeError as e:
                self.log("Cannot create hashable identifier for dictionary at index {0}: {1}. Error: {2}".format(
                    index, d, str(e)), "WARNING")
                # For unhashable values, fall back to string representation
                identifier = str(sorted(d.items()))

            if identifier not in seen:
                seen.add(identifier)
                unique_dicts.append(d)
                self.log("Added unique dictionary at index {0} to result list".format(index), "DEBUG")
            else:
                duplicates_found += 1
                self.log("Found duplicate dictionary at index {0} - skipping".format(index), "DEBUG")

        final_count = len(unique_dicts)

        if duplicates_found > 0:
            self.log("Deduplication completed: removed {0} duplicate(s) from {1} total items. Final count: {2}".format(
                duplicates_found, original_count, final_count), "INFO")
        else:
            self.log("Deduplication completed: no duplicates found in {0} items".format(
                original_count), "DEBUG")

        self.log("Deduplicated list result: {0}".format(self.pprint(unique_dicts)), "DEBUG")

        return unique_dicts



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: dns_services_count_info
short_description: Information module for Dns Services Count
description:
  - Get all Dns Services Count.
  - Retrieves the total number of DNS Services for given parameters.
  - For detailed information about the usage of the API, please refer to the Open API specification document
    https //github.com/cisco-en-programmability/catalyst-center-api-specs/blob/main/Assurance/CE_Cat_Center_Org-DNSServices-1.0.0-resolved.yaml.
version_added: '6.17.0'
extends_documentation_fragment:
  - cisco.dnac.module_info
author: Rafael Campos (@racampos)
options:
  headers:
    description: Additional headers.
    type: dict
  startTime:
    description:
      - >
        StartTime query parameter. Start time from which API queries the data set related to the resource. It
        must be specified in UNIX epochtime in milliseconds. Value is inclusive.
    type: float
  endTime:
    description:
      - >
        EndTime query parameter. End time to which API queries the data set related to the resource. It must be
        specified in UNIX epochtime in milliseconds. Value is inclusive.
    type: float
  serverIp:
    description:
      - >
        ServerIp query parameter. IP Address of the DNS Server. This parameter supports wildcard (`*`) character
        -based search. Example `10.76.81.*` or `*56.78*` or `*50.28` Examples serverIp=10.42.3.31 (single IP
        Address is requested) serverIp=10.42.3.31&serverIp=name2&fabricVnName=name3 (multiple IP Addresses are
        requested).
    type: str
  deviceId:
    description:
      - >
        DeviceId query parameter. The device UUID. Examples `deviceId=6bef213c-19ca-4170-8375-b694e251101c`
        (single deviceId is requested)
        `deviceId=6bef213c-19ca-4170-8375-b694e251101c&deviceId=32219612-819e-4b5e-a96b-cf22aca13dd9 (multiple
        networkDeviceIds with & separator).
    type: str
  deviceSiteHierarchyId:
    description:
      - >
        DeviceSiteHierarchyId query parameter. The full hierarchy breakdown of the site tree in id form starting
        from Global site UUID and ending with the specific site UUID. (Ex.
        `globalUuid/areaUuid/buildingUuid/floorUuid`) This field supports wildcard asterisk (`*`) character
        search support. E.g. `*uuid*, *uuid, uuid*` Examples
        `?deviceSiteHierarchyId=globalUuid/areaUuid/buildingUuid/floorUuid `(single siteHierarchyId requested) `
        ?deviceSiteHierarchyId=globalUuid/areaUuid/buildingUuid/floorUuid&deviceSiteHierarchyId=globalUuid/areaU
        uid2/buildingUuid2/floorUuid2` (multiple siteHierarchyIds requested).
    type: str
  deviceSiteId:
    description:
      - >
        DeviceSiteId query parameter. The UUID of the site. (Ex. `flooruuid`) Examples `?deviceSiteIds=id1`
        (single id requested) `?deviceSiteIds=id1&deviceSiteIds=id2&siteId=id3` (multiple ids requested).
    type: str
  ssid:
    description:
      - >
        Ssid query parameter. SSID is the name of wireless network to which client connects to. It is also
        referred to as WLAN ID - Wireless Local Area Network Identifier. This field supports wildcard (`*`)
        character-based search. If the field contains the (`*`) character, please use the /query API for search.
        Ex `*Alpha*` or `Alpha*` or `*Alpha` Examples `ssid=Alpha` (single ssid requested)
        `ssid=Alpha&ssid=Guest` (multiple ssid requested).
    type: str
requirements:
  - dnacentersdk >= 2.11.0
  - python >= 3.12
seealso:
  - name: Cisco DNA Center documentation for Devices RetrievesTheTotalNumberOfDNSServicesForGivenParameters
    description: Complete reference of the RetrievesTheTotalNumberOfDNSServicesForGivenParameters API.
    link: https://developer.cisco.com/docs/dna-center/#!retrieves-the-total-number-of-dns-services-for-given-parameters
notes:
  - SDK Method used are
    devices.Devices.retrieves_the_total_number_of_d_n_s_services_for_given_parameters,
  - Paths used are
    get /dna/data/api/v1/dnsServices/count,
"""

EXAMPLES = r"""
---
- name: Get all Dns Services Count
  cisco.dnac.dns_services_count_info:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    headers: "{{my_headers | from_json}}"
    startTime: 0
    endTime: 0
    serverIp: string
    deviceId: string
    deviceSiteHierarchyId: string
    deviceSiteId: string
    ssid: string
  register: result
"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "response": {
        "count": 0
      },
      "version": "string"
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: backups_info
short_description: Information module for Backups
description:
  - Get all Backups.
  - Get Backups by id.
  - This api is used to get a specific backup based on the provided `backup id`.
  - This api is used to get all the backup available in the configured storage.
version_added: '6.18.0'
extends_documentation_fragment:
  - cisco.dnac.module_info
author: Rafael Campos (@racampos)
options:
  headers:
    description: Additional headers.
    type: dict
  query:
    description:
      - Query query parameter. Filter based on the provided text on predefined fields.
    type: str
  offset:
    description:
      - Offset query parameter. The first record to show for this page.
    type: int
  limit:
    description:
      - Limit query parameter. The number of records to show for this page.
    type: int
  sortBy:
    description:
      - SortBy query parameter. A property within the response to sort by.
    type: str
  order:
    description:
      - >
        Order query parameter. Whether ascending or descending order should be used to sort the response.Use
        `asc` for ascending and `desc` for descending order .
    type: str
  id:
    description:
      - Id path parameter. The `id` of the backup to be retrieved.
    type: str
requirements:
  - dnacentersdk >= 2.11.0
  - python >= 3.12
seealso:
  - name: Cisco DNA Center documentation for Backup GetAllBackup
    description: Complete reference of the GetAllBackup API.
    link: https://developer.cisco.com/docs/dna-center/#!get-all-backup
  - name: Cisco DNA Center documentation for Backup GetBackupById
    description: Complete reference of the GetBackupById API.
    link: https://developer.cisco.com/docs/dna-center/#!get-backup-by-id
notes:
  - SDK Method used are
    backup.Backup.get_all_backup,
    backup.Backup.get_backup_by_id,
  - Paths used are
    get /dna/system/api/v1/backups,
    get /dna/system/api/v1/backups/{id},
"""

EXAMPLES = r"""
---
- name: Get all Backups
  cisco.dnac.backups_info:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    headers: "{{my_headers | from_json}}"
    query: string
    offset: 0
    limit: 0
    sortBy: string
    order: string
  register: result
- name: Get Backups by id
  cisco.dnac.backups_info:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    headers: "{{my_headers | from_json}}"
    id: string
  register: result
"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "response": {
        "compatibilityError": [
          {
            "endDate": "string",
            "namespace": "string",
            "response": {},
            "serviceName": "string",
            "startDate": "string"
          }
        ],
        "context": {
          "schedule": "string",
          "type": "string"
        },
        "createdBy": "string",
        "createdDate": "string",
        "duration": 0,
        "endDate": "string",
        "fipsEnabled": true,
        "id": "string",
        "installedPackages": [
          {
            "displayName": "string",
            "name": "string",
            "version": "string"
          }
        ],
        "internetProtocolVersion": "string",
        "isBackupAvailable": true,
        "isCompatible": true,
        "name": "string",
        "numberOfNodes": 0,
        "productType": "string",
        "productVersion": "string",
        "releaseDisplayName": "string",
        "releaseDisplayVersion": "string",
        "releaseName": "string",
        "releaseVersion": "string",
        "scope": "string",
        "size": 0,
        "status": "string",
        "storage": {
          "host": "string",
          "id": "string",
          "mountPath": "string",
          "name": "string",
          "serverPath": "string",
          "type": "string"
        },
        "versions": [
          "string"
        ]
      },
      "version": "string"
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: iot_rep_rings_id_info
short_description: Information module for Iot Rep Rings Id
description:
  - Get Iot Rep Rings Id by id. - > This API returns REP ring for the given id The id of configured REP ring can be retrieved
    using the API /dna/intent/api/v1/iot/repRings/query .
version_added: '6.18.0'
extends_documentation_fragment:
  - cisco.dnac.module_info
author: Rafael Campos (@racampos)
options:
  headers:
    description: Additional headers.
    type: dict
  id:
    description:
      - >
        Id path parameter. Ring ID of configured REP ring can be fetched using the API
        `/dna/intent/api/v1/iot/repRings/query`.
    type: str
requirements:
  - dnacentersdk >= 2.11.0
  - python >= 3.12
seealso:
  - name: Cisco DNA Center documentation for Industrial Configuration GetTheREPRingBasedOnTheRingId
    description: Complete reference of the GetTheREPRingBasedOnTheRingId API.
    link: https://developer.cisco.com/docs/dna-center/#!get-the-rep-ring-based-on-the-ring-id
notes:
  - SDK Method used are
    industrial_configuration.IndustrialConfiguration.get_the_r_e_p_ring_based_on_the_ring_id,
  - Paths used are
    get /dna/intent/api/v1/iot/repRings/{id},
"""

EXAMPLES = r"""
---
- name: Get Iot Rep Rings Id by id
  cisco.dnac.iot_rep_rings_id_info:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    headers: "{{my_headers | from_json}}"
    id: string
  register: result
"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: list
  elements: dict
  sample: >
    [
      {
        "response": [
          {
            "id": "string",
            "rootNetworkDeviceId": "string",
            "rootNeighbourNetworkDeviceIds": [
              "string"
            ],
            "status": "string",
            "repSegmentId": 0,
            "deploymentMode": "string",
            "ringName": "string",
            "ringMembers": [
              {
                "networkDeviceId": "string",
                "nodeName": "string",
                "portName1": "string",
                "portName2": "string",
                "ringOrder": 0
              }
            ]
          }
        ],
        "version": 0
      }
    ]
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: images_id_sites_site_id_untag_golden
short_description: Resource module for Images Id Sites Site Id Untag Golden
description:
  - Manage operation create of the resource Images Id Sites Site Id Untag Golden.
  - Untag the golden images specifically designed for a particular device type or supervisor engine module.
version_added: '6.17.0'
extends_documentation_fragment:
  - cisco.dnac.module
author: Rafael Campos (@racampos)
options:
  deviceRoles:
    description: Device Roles. Available value will be CORE, DISTRIBUTION, UNKNOWN, ACCESS, BORDER ROUTER.
    elements: str
    type: list
  deviceTags:
    description: Device tags can be fetched fom API https //developer.cisco.com/docs/dna-center/#!get-tag.
    elements: str
    type: list
  id:
    description: Id path parameter. Software image identifier is used for golden tagging or intent to tag it. The value of
      `id` can be obtained from the response of the API `/dna/intent/api/v1/images?imported=true&isAddonImages=false` for
      the base image and `/dna/images/{id}/addonImages` where `id` will be the software image identifier of the base image.
    type: str
  productNameOrdinal:
    description: The product name ordinal is a unique value for each network device product. `productNameOrdinal` can be obtained
      from the response of API `/dna/intent/api/v1/siteWiseProductNam...
    type: float
  siteId:
    description: SiteId path parameter. Site identifier for tagged image or intent to tag it. The default value is global
      site id. See https //developer.cisco.com/docs/dna-center(#!get-site) for `siteId`.
    type: str
  supervisorProductNameOrdinal:
    description: The supervisor engine module ordinal is a unique value for each supervisor module. `supervisorProductNameOrdinal`
      can be obtained from the response of API `/dna/intent/api/v1/siteWiseProductNames?siteId=<siteId>`.
    type: float
requirements:
  - dnacentersdk >= 2.11.0
  - python >= 3.12
seealso:
  - name: Cisco DNA Center documentation for Software Image Management (SWIM) UntaggingGoldenImage
    description: Complete reference of the UntaggingGoldenImage API.
    link: https://developer.cisco.com/docs/dna-center/#!untagging-golden-image
notes:
  - SDK Method used are
    software_image_management_swim.SoftwareImageManagementSwim.untagging_golden_image,
  - Paths used are
    post /dna/intent/api/v1/images/{id}/sites/{siteId}/untagGolden,
"""

EXAMPLES = r"""
---
- name: Create
  cisco.dnac.images_id_sites_site_id_untag_golden:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    deviceRoles:
      - string
    deviceTags:
      - string
    id: string
    productNameOrdinal: 0
    siteId: string
    supervisorProductNameOrdinal: 0
"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "response": {
        "taskId": "string",
        "url": "string"
      },
      "version": "string"
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: compliance_network_devices_detail_policys_info
short_description: Information module for Compliance Network Devices Detail Policys
description:
  - Get all Compliance Network Devices Detail Policys.
  - Retrieve the details of violating compliance policies for the network device.
version_added: '6.46.0'
extends_documentation_fragment:
  - cisco.dnac.module_info
author: Rafael Campos (@racampos)
options:
  headers:
    description: Additional headers.
    type: dict
  networkDeviceId:
    description:
      - NetworkDeviceId path parameter. The `id` of the network device.
    type: str
  offset:
    description:
      - Offset query parameter. The first record to show for this page; the first record is numbered 1.
    type: int
  limit:
    description:
      - >
        Limit query parameter. The number of records to show for this page. Default is 500 if not specified.
        Maximum allowed limit is 500.
    type: int
requirements:
  - dnacentersdk >= 2.11.0
  - python >= 3.12
seealso:
  - name: Cisco DNA Center documentation for Compliance RetrieveTheViolatingPolicies
    description: Complete reference of the RetrieveTheViolatingPolicies API.
    link: https://developer.cisco.com/docs/dna-center/#!retrieve-the-violating-policies
notes:
  - SDK Method used are
    compliance.Compliance.retrieve_the_violating_policies,
  - Paths used are
    get /dna/intent/api/v1/compliance/networkDevices/{networkDeviceId}/detail/policys,
"""

EXAMPLES = r"""
---
- name: Get all Compliance Network Devices Detail Policys
  cisco.dnac.compliance_network_devices_detail_policys_info:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    headers: "{{my_headers | from_json}}"
    offset: 0
    limit: 0
    networkDeviceId: string
  register: result
"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "response": [
        {
          "policyId": "string",
          "policyName": "string",
          "policyDescription": "string",
          "maxSeverity": "string",
          "violationsCount": 0,
          "errorCount": 0,
          "lastComplianceRunTime": 0
        }
      ],
      "version": "string"
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: network_devices_device_controllability_settings
short_description: Resource module for Network Devices Device Controllability Settings
description:
  - Manage operation update of the resource Network Devices Device Controllability Settings.
  - Device Controllability is a system-level process on Catalyst Center that enforces state.
version_added: '6.15.0'
extends_documentation_fragment:
  - cisco.dnac.module
author: Rafael Campos (@racampos)
options:
  autocorrectTelemetryConfig:
    description: If it is true, autocorrect telemetry config is enabled. If it is false, autocorrect telemetry config is disabled.
      The autocorrect telemetry config feature is supported only when device controllability is enabled.
    type: bool
  deviceControllability:
    description: If it is true, device controllability is enabled. If it is false, device controllability is disabled.
    type: bool
requirements:
  - dnacentersdk >= 2.11.0
  - python >= 3.12
seealso:
  - name: Cisco DNA Center documentation for Site Design UpdateDeviceControllabilitySettings
    description: Complete reference of the UpdateDeviceControllabilitySettings API.
    link: https://developer.cisco.com/docs/dna-center/#!update-device-controllability-settings
notes:
  - SDK Method used are
    site_design.SiteDesign.update_device_controllability_settings,
  - Paths used are
    put /dna/intent/api/v1/networkDevices/deviceControllability/settings,
"""

EXAMPLES = r"""
---
- name: Update all
  cisco.dnac.network_devices_device_controllability_settings:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    state: present
    autocorrectTelemetryConfig: true
    deviceControllability: true
"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "version": "string",
      "response": {
        "count": 0
      }
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: interfaces_id_trend_analytics
short_description: Resource module for Interfaces Id Trend Analytics
description:
  - Manage operation create of the resource Interfaces Id Trend Analytics. - > The Trend analytcis data for the interface,
    identified by its instanceUuid, in the specified time range. The data is grouped based on the trend time Interval, other
    input parameters like attributes and aggregate attributes. The default time interval range is 3 hours when start and endTime
    is not provided.
version_added: '6.17.0'
extends_documentation_fragment:
  - cisco.dnac.module
author: Rafael Campos (@racampos)
options:
  aggregateAttributes:
    description: Interfaces Id Trend Analytics's aggregateAttributes.
    elements: dict
    suboptions:
      function:
        description: Function.
        type: str
      name:
        description: Name.
        type: str
    type: list
  attributes:
    description: Attributes.
    elements: str
    type: list
  endTime:
    description: End Time.
    type: int
  filters:
    description: Interfaces Id Trend Analytics's filters.
    elements: dict
    suboptions:
      key:
        description: Key.
        type: str
      operator:
        description: Operator.
        type: str
      value:
        description: Value.
        type: str
    type: list
  id:
    description: Id path parameter. The interface instance Uuid.
    type: str
  startTime:
    description: Start Time.
    type: int
  timestampOrder:
    description: Timestamp Order.
    type: str
  trendIntervalInMinutes:
    description: Trend Interval In Minutes.
    type: int
requirements:
  - dnacentersdk >= 2.11.0
  - python >= 3.12
seealso:
  - name: Cisco DNA Center documentation for Devices TheTrendAnalytcisDataForTheInterfacesInTheSpecifiedTimeRange
    description: Complete reference of the TheTrendAnalytcisDataForTheInterfacesInTheSpecifiedTimeRange API.
    link: https://developer.cisco.com/docs/dna-center/#!the-trend-analytcis-data-for-the-interfaces-in-the-specified-time-range
notes:
  - SDK Method used are
    devices.Devices.the_trend_analytcis_data_for_the_interfaces_in_the_specified_time_range,
  - Paths used are
    post /dna/data/api/v1/interfaces/{id}/trendAnalytics,
"""

EXAMPLES = r"""
---
- name: Create
  cisco.dnac.interfaces_id_trend_analytics:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    aggregateAttributes:
      - function: string
        name: string
    attributes:
      - string
    endTime: 0
    filters:
      - key: string
        operator: string
        value: string
    id: string
    startTime: 0
    timestampOrder: string
    trendIntervalInMinutes: 0
"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "response": [
        {
          "timestamp": 0,
          "attributes": [
            {
              "name": "string",
              "value": "string"
            }
          ],
          "aggregateAttributes": [
            {
              "name": "string"
            }
          ]
        }
      ],
      "timestampOrder": "string"
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: global_credential_update
short_description: Resource module for Global Credential Update
description:
  - Manage operation update of the resource Global Credential Update.
  - Update global credential for network devices in sites.
version_added: '3.1.0'
extends_documentation_fragment:
  - cisco.dnac.module
author: Rafael Campos (@racampos)
options:
  globalCredentialId:
    description: GlobalCredentialId path parameter. Global credential Uuid.
    type: str
  siteUuids:
    description: List of siteUuids where credential is to be updated.
    elements: str
    type: list
requirements:
  - dnacentersdk >= 2.11.0
  - python >= 3.12
seealso:
  - name: Cisco DNA Center documentation for Discovery UpdateGlobalCredentials
    description: Complete reference of the UpdateGlobalCredentials API.
    link: https://developer.cisco.com/docs/dna-center/#!update-global-credentials
notes:
  - SDK Method used are
    discovery.Discovery.update_global_credentials,
  - Paths used are
    put /dna/intent/api/v1/global-credential/{globalCredentialId},
"""

EXAMPLES = r"""
---
- name: Update by id
  cisco.dnac.global_credential_update:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    globalCredentialId: string
    siteUuids:
      - string
"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "response": {
        "taskId": "string",
        "url": "string"
      },
      "version": "string"
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: site_wise_product_names_info
short_description: Information module for Site Wise Product Names
description:
  - Get all Site Wise Product Names. - > Provides network device product names for a site. The default value of `siteId` is
    global. The response will include the network device count and image summary.
version_added: '6.15.0'
extends_documentation_fragment:
  - cisco.dnac.module_info
author: Rafael Campos (@racampos)
options:
  headers:
    description: Additional headers.
    type: dict
  siteId:
    description:
      - >
        SiteId query parameter. Site identifier to get the list of all available products under the site. The
        default value is the global site. See https //developer.cisco.com/docs/dna-center/get-site for siteId.
    type: str
  productName:
    description:
      - >
        ProductName query parameter. Filter with network device product name. Supports partial case-insensitive
        search. A minimum of 3 characters are required for search.
    type: str
  offset:
    description:
      - >
        Offset query parameter. The first record to show for this page; the first record is numbered 1. The
        minimum value is 1.
    type: int
  limit:
    description:
      - >
        Limit query parameter. The number of records to show for this page. The minimum and maximum values are 1
        and 500, respectively.
    type: int
requirements:
  - dnacentersdk >= 2.11.0
  - python >= 3.12
seealso:
  - name: Cisco DNA Center documentation for Software Image Management (SWIM) ReturnsNetworkDeviceProductNamesForASite
    description: Complete reference of the ReturnsNetworkDeviceProductNamesForASite API.
    link: https://developer.cisco.com/docs/dna-center/#!returns-network-device-product-names-for-a-site
notes:
  - SDK Method used are
    software_image_management_swim.SoftwareImageManagementSwim.returns_network_device_product_names_for_a_site,
  - Paths used are
    get /dna/intent/api/v1/siteWiseProductNames,
"""

EXAMPLES = r"""
---
- name: Get all Site Wise Product Names
  cisco.dnac.site_wise_product_names_info:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    headers: "{{my_headers | from_json}}"
    siteId: string
    productName: string
    offset: 0
    limit: 0
  register: result
"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "response": [
        {
          "id": "string",
          "productNameOrdinal": 0,
          "productName": "string",
          "supervisorProductName": "string",
          "supervisorProductNameOrdinal": 0,
          "networkDeviceCount": 0,
          "imageSummary": {
            "installedImageCount": 0,
            "goldenImageCount": 0,
            "installedImageAdvisorCount": 0
          }
        }
      ],
      "version": "string"
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: integrate_ise
short_description: Resource module for Integrate Ise
description:
  - Manage operation update of the resource Integrate Ise. - > API to accept Cisco ISE server certificate for Cisco ISE server
    integration. Use 'Cisco ISE Server Integration Status' Intent API to check the integration status. This API can be used
    to retry the failed integration.
version_added: '6.14.0'
extends_documentation_fragment:
  - cisco.dnac.module
author: Rafael Campos (@racampos)
options:
  id:
    description: Id path parameter. Cisco ISE Server Identifier. Use 'Get Authentication and Policy Servers' intent API to
      find the identifier.
    type: str
  isCertAcceptedByUser:
    description: Value true for accept, false for deny. Remove this field and send empty request payload ( {} ) to retry the
      failed integration.
    type: bool
requirements:
  - dnacentersdk >= 2.11.0
  - python >= 3.12
seealso:
  - name: Cisco DNA Center documentation for System Settings AcceptCiscoISEServerCertificateForCiscoISEServerIntegration
    description: Complete reference of the AcceptCiscoISEServerCertificateForCiscoISEServerIntegration API.
    link: https://developer.cisco.com/docs/dna-center/#!accept-cisco-ise-server-certificate-for-cisco-ise-server-integration
notes:
  - SDK Method used are
    system_settings.SystemSettings.accept_cisco_ise_server_certificate_for_cisco_ise_server_integration,
  - Paths used are
    put /dna/intent/api/v1/integrate-ise/{id},
"""

EXAMPLES = r"""
---
- name: Update by id
  cisco.dnac.integrate_ise:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    id: string
    isCertAcceptedByUser: true
"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {}
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: wireless_controllers_certificate_renewal_create
short_description: Resource module for Wireless Controllers Certificate Renewal Create
description:
  - Manage operation create of the resource Wireless Controllers Certificate Renewal Create.
  - This API allows user to renew LSC certificates of access points.
version_added: '6.46.0'
extends_documentation_fragment:
  - cisco.dnac.module
author: Rafael Campos (@racampos)
options:
  accessPointMacs:
    description: MAC address of access points on which certificate renewal has to be performed.
    elements: str
    type: list
  deviceId:
    description:
      - DeviceId path parameter. Network Device ID.
      - This value can be obtained by using the API call GET /dna/intent/api/v1/network-device/ip-address/${ipAddress}.
    type: str
  expiryTime:
    description: Expiry time with in which access points certificates are set to expire.
    type: int
  siteTagIds:
    description: Site tag IDs on which LSC instantaneous certificate renewal has to be performed.
    elements: str
    type: list
requirements:
  - dnacentersdk >= 2.11.0
  - python >= 3.12
seealso:
  - name: Cisco DNA Center documentation for Wireless AccessPointsLSCCertificateInstantaneousRenewal
    description: Complete reference of the AccessPointsLSCCertificateInstantaneousRenewal API.
    link: https://developer.cisco.com/docs/dna-center/#!access-points-lsc-certificate-instantaneous-renewal
notes:
  - SDK Method used are
    wireless.Wireless.access_points_l_s_c_certificate_instantaneous_renewal,
  - Paths used are
    post /dna/intent/api/v1/wirelessControllers/{deviceId}/certificateRenewal,
"""

EXAMPLES = r"""
---
- name: Create
  cisco.dnac.wireless_controllers_certificate_renewal_create:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    accessPointMacs:
      - string
    deviceId: string
    expiryTime: 0
    siteTagIds:
      - string
"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "response": {
        "taskId": "string",
        "url": "string"
      },
      "version": "string"
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: site_kpi_summaries_top_n_analytics
short_description: Resource module for Site Kpi Summaries Top N Analytics
description:
  - Manage operation create of the resource Site Kpi Summaries Top N Analytics. - > Gets the Top N entites related based on
    site analytics for a given kpi type. For detailed information about the usage of the API, please refer to the Open API
    specification document - https //github.com/cisco-en- programmability/catalyst-center-api-specs/blob/main/Assurance/CE_Cat_Center_Org-
    SiteKpiSummaries-1.0.0-resolved.yaml.
version_added: '6.17.0'
extends_documentation_fragment:
  - cisco.dnac.module
author: Rafael Campos (@racampos)
options:
  endTime:
    description: End Time.
    type: int
  filters:
    description: Site Kpi Summaries Top N Analytics's filters.
    elements: dict
    suboptions:
      key:
        description: Key.
        type: str
      operator:
        description: Operator.
        type: str
      value:
        description: Value.
        type: str
    type: list
  groupBy:
    description: Group By.
    elements: str
    type: list
  headers:
    description: Additional headers.
    type: dict
  startTime:
    description: Start Time.
    type: int
  topN:
    description: Top N.
    type: int
requirements:
  - dnacentersdk >= 2.11.0
  - python >= 3.12
seealso:
  - name: Cisco DNA Center documentation for Sites SubmitRequestForTopNEntitiesRelatedToSiteAnalytics
    description: Complete reference of the SubmitRequestForTopNEntitiesRelatedToSiteAnalytics API.
    link: https://developer.cisco.com/docs/dna-center/#!submit-request-for-top-n-entities-related-to-site-analytics
notes:
  - SDK Method used are
    sites.Sites.submit_request_for_top_n_entities_related_to_site_analytics,
  - Paths used are
    post /dna/data/api/v1/siteKpiSummaries/topNAnalytics,
"""

EXAMPLES = r"""
---
- name: Create
  cisco.dnac.site_kpi_summaries_top_n_analytics:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    state: present
    endTime: 0
    filters:
      - key: string
        operator: string
        value: string
    groupBy:
      - string
    headers: '{{my_headers | from_json}}'
    startTime: 0
    topN: 0
"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "response": {
        "taskLocation": "string",
        "taskId": "string"
      },
      "version": "string"
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
module: networks_pii_sm_devices_for_key_info
short_description: Information module for networks _pii _smdevicesforkey
description:
  - Information module for Networks Pii Smdevicesforkey Info.
  - Get all networks _pii _smdevicesforkey.
  - >
    Given a piece of Personally Identifiable Information PII , return the Systems
    Manager device IDs associated with that identifier. These device IDs can be used
    with the Systems Manager API endpoints to retrieve device details. Exactly one
    identifier will be accepted.
version_added: '1.0.0'
extends_documentation_fragment:
  - cisco.meraki.module_info
author: Francisco Munoz (@fmunoz)
options:
  headers:
    description: Additional headers.
    type: dict
  networkId:
    description:
      - Information module for Networks Pii Smdevicesforkey Info.
      - NetworkId path parameter. Network ID.
    type: str
  username:
    description:
      - Information module for Networks Pii Smdevicesforkey Info.
      - Username query parameter. The username of a Systems Manager user.
    type: str
  email:
    description:
      - Information module for Networks Pii Smdevicesforkey Info.
      - Email query parameter. The email of a network user account or a Systems Manager
        device.
    type: str
  mac:
    description:
      - Information module for Networks Pii Smdevicesforkey Info.
      - Mac query parameter. The MAC of a network client device or a Systems Manager
        device.
    type: str
  serial:
    description:
      - Information module for Networks Pii Smdevicesforkey Info.
      - Serial query parameter. The serial of a Systems Manager device.
    type: str
  imei:
    description:
      - Information module for Networks Pii Smdevicesforkey Info.
      - Imei query parameter. The IMEI of a Systems Manager device.
    type: str
  bluetoothMac:
    description:
      - Information module for Networks Pii Smdevicesforkey Info.
      - BluetoothMac query parameter. The MAC of a Bluetooth client.
    type: str
requirements:
  - meraki >= 2.4.9
  - python >= 3.5
seealso:
  - name: Cisco Meraki documentation for networks getNetworkPiiSmDevicesForKey
    description: Complete reference of the getNetworkPiiSmDevicesForKey API.
    link: https://developer.cisco.com/meraki/api-v1/#!get-network-pii-sm-devices-for-key
notes:
  - SDK Method used are
    networks.Networks.get_network_pii_sm_devices_for_key,
  - Paths used are
    get /networks/{networkId}/pii/smDevicesForKey,
"""

EXAMPLES = r"""
- name: Get all networks _pii _smdevicesforkey
  cisco.meraki.networks_pii_sm_devices_for_key_info:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    username: string
    email: string
    mac: string
    serial: string
    imei: string
    bluetoothMac: string
    networkId: string
  register: result
"""
RETURN = r"""
meraki_response:
  description: A dictionary or list with the response returned by the Cisco Meraki Python SDK
  returned: always
  type: dict
  sample:
  - {}
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: sites_wireless_settings_ssids
short_description: Resource module for Sites Wireless Settings Ssids
description:
  - Manage operations create, update and delete of the resource Sites Wireless Settings Ssids.
  - This API allows the user to create an SSID Service Set Identifier at the Global site. - > This API allows the user to
    delete an SSID Service Set Identifier at the global level , if the SSID is not mapped to any Wireless Profile, Or remove
    override from given site Id .
  - This API allows the user to update an SSID Service Set Identifier at the given site.
version_added: '6.15.0'
extends_documentation_fragment:
  - cisco.dnac.module
author: Rafael Campos (@racampos)
options:
  aaaOverride:
    description: Activate the AAA Override feature when set to true.
    type: bool
  acctServers:
    description: List of Accounting server IpAddresses.
    elements: str
    type: list
  aclName:
    description: Pre-Auth Access Control List (ACL) Name.
    type: str
  authServer:
    description: For Guest SSIDs ('wlanType' is 'Guest' and 'l3AuthType' is 'web_auth'), the Authentication Server('authServer')
      is mandatory. Otherwise, it defaults to 'auth_external'.
    type: str
  authServers:
    description: List of Authentication/Authorization server IpAddresses.
    elements: str
    type: list
  authType:
    description: L2 Authentication Type (If authType is not open , then atleast one RSN Cipher Suite and corresponding valid
      AKM must be enabled). Default is L2 Authentication Type if exists else None.
    type: str
  basicServiceSetClientIdleTimeout:
    description: This refers to the duration of inactivity, measured in seconds, before a client connected to the Basic Service
      Set is considered idle and timed out. Default is Basic ServiceSet ClientIdle Timeout if exists else 300. If it needs
      to be disabled , pass 0 as its value else valid range is 15 to 100000.
    type: int
  basicServiceSetMaxIdleEnable:
    description: Activate the maximum idle feature for the Basic Service Set.
    type: bool
  cckmTsfTolerance:
    description: He default value is the Cckm Timestamp Tolerance (in milliseconds, if specified); otherwise, it is 0.
    type: int
  clientExclusionEnable:
    description: Activate the feature that allows for the exclusion of clients.
    type: bool
  clientExclusionTimeout:
    description: This refers to the length of time, in seconds, a client is excluded or blocked from accessing the network
      after a specified number of unsuccessful attempts. Default is Client Exclusion Timeout if exists else 180.
    type: int
  clientRateLimit:
    description: This pertains to the maximum data transfer rate, specified in bits per second, that a client is permitted
      to achieve. It should be in mutliples of 500. Default is Client Rate Limit if exists else 0.
    type: int
  coverageHoleDetectionEnable:
    description: Activate Coverage Hole Detection feature when set to true.
    type: bool
  directedMulticastServiceEnable:
    description: The Directed Multicast Service feature becomes operational when it is set to true.
    type: bool
  egressQos:
    description: Egress QOS.
    type: str
  externalAuthIpAddress:
    description: External WebAuth URL (Mandatory for Guest SSIDs with wlanType = Guest, l3AuthType = web_auth and authServer
      = auth_external).
    type: str
  fastTransition:
    description: Fast Transition.
    type: str
  fastTransitionOverTheDistributedSystemEnable:
    description: Enable Fast Transition over the Distributed System when set to true.
    type: bool
  ghz24Policy:
    description: 2.4 Ghz Band Policy value. Allowed only when 2.4 Radio Band is enabled in ssidRadioType.
    type: str
  ghz6PolicyClientSteering:
    description: true if 6 GHz Policy Client Steering is enabled, else False.
    type: bool
  id:
    description: Id path parameter. SSID ID.
    type: str
  ingressQos:
    description: Ingress QOS.
    type: str
  isApBeaconProtectionEnabled:
    description: When set to true, the Access Point (AP) Beacon Protection feature is activated, enhancing the security of
      the network.
    type: bool
  isAuthKey8021x:
    description: When set to true, the 802.1X authentication key is in use.
    type: bool
  isAuthKey8021xPlusFT:
    description: When set to true, the 802.1X-Plus-FT authentication key is in use.
    type: bool
  isAuthKey8021x_SHA256:
    description: When set to true, the feature that enables 802.1X authentication using the SHA256 algorithm is turned on.
    type: bool
  isAuthKeyEasyPSK:
    description: When set to true, the feature that enables the use of Easy Pre-shared Key (PSK) authentication is activated.
    type: bool
  isAuthKeyOWE:
    description: When set to true, the Opportunistic Wireless Encryption (OWE) authentication key feature is turned on.
    type: bool
  isAuthKeyPSK:
    description: When set to true, the Pre-shared Key (PSK) authentication feature is enabled.
    type: bool
  isAuthKeyPSKPlusFT:
    description: When set to true, the feature that enables the combination of Pre-shared Key (PSK) and Fast Transition (FT)
      authentication keys is activated.
    type: bool
  isAuthKeyPSKSHA256:
    description: The feature that allows the use of Pre-shared Key (PSK) authentication with the SHA256 algorithm is enabled
      when it is set to true.
    type: bool
  isAuthKeySae:
    description: When set to true, the feature enabling the Simultaneous Authentication of Equals (SAE) authentication key
      is activated.
    type: bool
  isAuthKeySaeExt:
    description: When set to true, the Simultaneous Authentication of Equals (SAE) Extended Authentication key feature is
      turned on.
    type: bool
  isAuthKeySaeExtPlusFT:
    description: When set to true, the Simultaneous Authentication of Equals (SAE) combined with Fast Transition (FT) Authentication
      Key feature is enabled.
    type: bool
  isAuthKeySaePlusFT:
    description: Activating this setting by switching it to true turns on the authentication key feature that supports both
      Simultaneous Authentication of Equals (SAE) and Fast Transition (FT).
    type: bool
  isAuthKeySuiteB1921x:
    description: When set to true, the SuiteB192-1x authentication key feature is enabled.
    type: bool
  isAuthKeySuiteB1x:
    description: When activated by setting it to true, the SuiteB-1x authentication key feature is engaged.
    type: bool
  isBroadcastSSID:
    description: When activated by setting it to true, the Broadcast SSID feature will make the SSID publicly visible to wireless
      devices searching for available networks.
    type: bool
  isCckmEnabled:
    description: true if CCKM is enabled, else False.
    type: bool
  isEnabled:
    description: Set SSID's admin status as 'Enabled' when set to true.
    type: bool
  isFastLaneEnabled:
    description: true if FastLane is enabled, else False.
    type: bool
  isHex:
    description: true if passphrase is in Hex format, else False.
    type: bool
  isMacFilteringEnabled:
    description: When set to true, MAC Filtering will be activated, allowing control over network access based on the MAC
      address of the device.
    type: bool
  isPosturingEnabled:
    description: Applicable only for Enterprise SSIDs. When set to True, Posturing will enabled. Required to be set to True
      if ACL needs to be mapped for Enterprise SSID.
    type: bool
  isRadiusProfilingEnabled:
    description: '''true'' if Radius profiling needs to be enabled, defaults to ''false'' if not specified. At least one AAA/PSN
      server is required to enable Radius Profiling.'
    type: bool
  isRandomMacFilterEnabled:
    description: Deny clients using randomized MAC addresses when set to true.
    type: bool
  l3AuthType:
    description: "L3 Authentication Type. When 'wlanType' is 'Enterprise', 'l3AuthType' is optional and defaults to 'open'
      if not specified. If 'wlanType' is 'Guest' then 'l3AuthType' is mandatory."
    type: str
  managementFrameProtectionClientprotection:
    description: Default is Management Frame Protection Client if exists else Optional.
    type: str
  multiPSKSettings:
    description: Sites Wireless Settings Ssids's multiPSKSettings.
    elements: dict
    suboptions:
      passphrase:
        description: Passphrase needs to be between 8 and 63 characters for ASCII type. HEX passphrase needs to be 64 characters.
        type: str
      passphraseType:
        description: Passphrase Type(default ASCII).
        type: str
      priority:
        description: Priority.
        type: int
    type: list
  nasOptions:
    description: Pre-Defined NAS Options AP ETH Mac Address, AP IP address, AP Location , AP MAC Address, AP Name, AP Policy
      Tag, AP Site Tag, SSID, System IP Address, System MAC Address, System Name.
    elements: str
    type: list
  neighborListEnable:
    description: The Neighbor List feature is enabled when it is set to true.
    type: bool
  openSsid:
    description: Open SSID which is already created in the design and not associated to any other OPEN-SECURED SSID.
    type: str
  passphrase:
    description: Passphrase (Only applicable for SSID with PERSONAL security level). Passphrase needs to be between 8 and
      63 characters for ASCII type. HEX passphrase needs to be 64 characters.
    type: str
  policyProfileName:
    description: "Policy Profile Name. If 'policyProfileName' is not provided, the value of 'profileName' will be assigned
      to it. If 'profileName' is also not provided, an autogenerated name will be used. Autogenerated name is generated by
      appending 'ssid' field's value with '_profile' (Example If 'ssid' = 'ExampleSsid', then autogenerated name will be 'ExampleSsid_profile')."
    type: str
  profileName:
    description: WLAN Profile Name, if not passed autogenerated profile name will be assigned.
    type: str
  protectedManagementFrame:
    description: (REQUIRED is applicable for authType WPA3_PERSONAL, WPA3_ENTERPRISE, OPEN_SECURED) and (OPTIONAL/REQUIRED
      is applicable for authType WPA2_WPA3_PERSONAL and WPA2_WPA3_ENTERPRISE).
    type: str
  removeOverrideInHierarchy:
    description: RemoveOverrideInHierarchy query parameter. Remove override in hierarchy. Refer Feature tab for details.
    type: bool
  rsnCipherSuiteCcmp128:
    description: When set to true, the Robust Security Network (RSN) Cipher Suite CCMP128 encryption protocol is activated.
    type: bool
  rsnCipherSuiteCcmp256:
    description: When set to true, the Robust Security Network (RSN) Cipher Suite CCMP256 encryption protocol is activated.
    type: bool
  rsnCipherSuiteGcmp128:
    description: When set to true, the Robust Security Network (RSN) Cipher Suite GCMP128 encryption protocol is activated.
    type: bool
  rsnCipherSuiteGcmp256:
    description: When set to true, the Robust Security Network (RSN) Cipher Suite GCMP256 encryption protocol is activated.
    type: bool
  sessionTimeOut:
    description: This denotes the allotted time span, expressed in seconds, before a session is automatically terminated due
      to inactivity. Default sessionTimeOut is 1800.
    type: int
  sessionTimeOutEnable:
    description: Turn on the feature that imposes a time limit on user sessions.
    type: bool
  siteId:
    description: SiteId path parameter. Site UUID of Global site.
    type: str
  sleepingClientEnable:
    description: When set to true, this will activate the timeout settings that apply to clients in sleep mode.
    type: bool
  sleepingClientTimeout:
    description: This refers to the amount of time, measured in minutes, before a sleeping (inactive) client is timed out
      of the network. Default is Sleeping Client Timeout if exists else 720.
    type: int
  ssid:
    description: Name of the SSID.
    type: str
  ssidRadioType:
    description: Radio Policy Enum (default Triple band operation(2.4GHz, 5GHz and 6GHz)).
    type: str
  webPassthrough:
    description: When set to true, the Web-Passthrough feature will be activated for the Guest SSID, allowing guests to bypass
      certain login requirements.
    type: bool
  wlanBandSelectEnable:
    description: Band select is allowed only when band options selected contains at least 2.4 GHz and 5 GHz band else false.
    type: bool
  wlanType:
    description: Wlan Type.
    type: str
requirements:
  - dnacentersdk >= 2.11.0
  - python >= 3.12
seealso:
  - name: Cisco DNA Center documentation for Wireless CreateSSID
    description: Complete reference of the CreateSSID API.
    link: https://developer.cisco.com/docs/dna-center/#!create-ssid
  - name: Cisco DNA Center documentation for Wireless DeleteSSID
    description: Complete reference of the DeleteSSID API.
    link: https://developer.cisco.com/docs/dna-center/#!delete-ssid
  - name: Cisco DNA Center documentation for Wireless UpdateSSID
    description: Complete reference of the UpdateSSID API.
    link: https://developer.cisco.com/docs/dna-center/#!update-ssid
notes:
  - SDK Method used are
    wireless.Wireless.create_ssid,
    wireless.Wireless.delete_ssid,
    wireless.Wireless.update_ssid,
  - Paths used are
    post /dna/intent/api/v1/sites/{siteId}/wirelessSettings/ssids,
    delete /dna/intent/api/v1/sites/{siteId}/wirelessSettings/ssids/{id},
    put /dna/intent/api/v1/sites/{siteId}/wirelessSettings/ssids/{id},
"""

EXAMPLES = r"""
---
- name: Create
  cisco.dnac.sites_wireless_settings_ssids:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    state: present
    aaaOverride: true
    acctServers:
      - string
    aclName: string
    authServer: string
    authServers:
      - string
    authType: string
    basicServiceSetClientIdleTimeout: 0
    basicServiceSetMaxIdleEnable: true
    cckmTsfTolerance: 0
    clientExclusionEnable: true
    clientExclusionTimeout: 0
    clientRateLimit: 0
    coverageHoleDetectionEnable: true
    directedMulticastServiceEnable: true
    egressQos: string
    externalAuthIpAddress: string
    fastTransition: string
    fastTransitionOverTheDistributedSystemEnable: true
    ghz24Policy: string
    ghz6PolicyClientSteering: true
    ingressQos: string
    isApBeaconProtectionEnabled: true
    isAuthKey8021x: true
    isAuthKey8021xPlusFT: true
    isAuthKey8021x_SHA256: true
    isAuthKeyEasyPSK: true
    isAuthKeyOWE: true
    isAuthKeyPSK: true
    isAuthKeyPSKPlusFT: true
    isAuthKeyPSKSHA256: true
    isAuthKeySae: true
    isAuthKeySaeExt: true
    isAuthKeySaeExtPlusFT: true
    isAuthKeySaePlusFT: true
    isAuthKeySuiteB1921x: true
    isAuthKeySuiteB1x: true
    isBroadcastSSID: true
    isCckmEnabled: true
    isEnabled: true
    isFastLaneEnabled: true
    isHex: true
    isMacFilteringEnabled: true
    isPosturingEnabled: true
    isRadiusProfilingEnabled: true
    isRandomMacFilterEnabled: true
    l3AuthType: string
    managementFrameProtectionClientprotection: string
    multiPSKSettings:
      - passphrase: string
        passphraseType: string
        priority: 0
    nasOptions:
      - string
    neighborListEnable: true
    openSsid: string
    passphrase: string
    policyProfileName: string
    profileName: string
    protectedManagementFrame: string
    rsnCipherSuiteCcmp128: true
    rsnCipherSuiteCcmp256: true
    rsnCipherSuiteGcmp128: true
    rsnCipherSuiteGcmp256: true
    sessionTimeOut: 0
    sessionTimeOutEnable: true
    siteId: string
    sleepingClientEnable: true
    sleepingClientTimeout: 0
    ssid: string
    ssidRadioType: string
    webPassthrough: true
    wlanBandSelectEnable: true
    wlanType: string
- name: Delete by id
  cisco.dnac.sites_wireless_settings_ssids:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    state: absent
    id: string
    removeOverrideInHierarchy: true
    siteId: string
- name: Update by id
  cisco.dnac.sites_wireless_settings_ssids:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    state: present
    aaaOverride: true
    acctServers:
      - string
    aclName: string
    authServer: string
    authServers:
      - string
    authType: string
    basicServiceSetClientIdleTimeout: 0
    basicServiceSetMaxIdleEnable: true
    cckmTsfTolerance: 0
    clientExclusionEnable: true
    clientExclusionTimeout: 0
    clientRateLimit: 0
    coverageHoleDetectionEnable: true
    directedMulticastServiceEnable: true
    egressQos: string
    externalAuthIpAddress: string
    fastTransition: string
    fastTransitionOverTheDistributedSystemEnable: true
    ghz24Policy: string
    ghz6PolicyClientSteering: true
    id: string
    ingressQos: string
    isApBeaconProtectionEnabled: true
    isAuthKey8021x: true
    isAuthKey8021xPlusFT: true
    isAuthKey8021x_SHA256: true
    isAuthKeyEasyPSK: true
    isAuthKeyOWE: true
    isAuthKeyPSK: true
    isAuthKeyPSKPlusFT: true
    isAuthKeyPSKSHA256: true
    isAuthKeySae: true
    isAuthKeySaeExt: true
    isAuthKeySaeExtPlusFT: true
    isAuthKeySaePlusFT: true
    isAuthKeySuiteB1921x: true
    isAuthKeySuiteB1x: true
    isBroadcastSSID: true
    isCckmEnabled: true
    isEnabled: true
    isFastLaneEnabled: true
    isHex: true
    isMacFilteringEnabled: true
    isPosturingEnabled: true
    isRadiusProfilingEnabled: true
    isRandomMacFilterEnabled: true
    l3AuthType: string
    managementFrameProtectionClientprotection: string
    multiPSKSettings:
      - passphrase: string
        passphraseType: string
        priority: 0
    nasOptions:
      - string
    neighborListEnable: true
    openSsid: string
    passphrase: string
    policyProfileName: string
    profileName: string
    protectedManagementFrame: string
    rsnCipherSuiteCcmp128: true
    rsnCipherSuiteCcmp256: true
    rsnCipherSuiteGcmp128: true
    rsnCipherSuiteGcmp256: true
    sessionTimeOut: 0
    sessionTimeOutEnable: true
    siteId: string
    sleepingClientEnable: true
    sleepingClientTimeout: 0
    ssid: string
    ssidRadioType: string
    webPassthrough: true
    wlanBandSelectEnable: true
    wlanType: string
"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "response": {
        "taskId": "string",
        "url": "string"
      },
      "version": "string"
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: site_kpi_summaries_query
short_description: Resource module for Site Kpi Summaries Query
description:
  - Manage operation create of the resource Site Kpi Summaries Query. - > Returns site analytics for all child sites of given
    parent site. For detailed information about the usage of the API, please refer to the Open API specification document
    - https //github.com/cisco-en- programmability/catalyst-center-api-specs/blob/main/Assurance/CE_Cat_Center_Org- SiteKpiSummaries-1.0.0-resolved.yaml.
version_added: '6.17.0'
extends_documentation_fragment:
  - cisco.dnac.module
author: Rafael Campos (@racampos)
options:
  attributes:
    description: Attributes.
    elements: str
    type: list
  endTime:
    description: End Time.
    type: int
  filters:
    description: Site Kpi Summaries Query's filters.
    elements: dict
    suboptions:
      key:
        description: Key.
        type: str
      operator:
        description: Operator.
        type: str
      value:
        description: Value.
        type: str
    type: list
  headers:
    description: Additional headers.
    type: dict
  page:
    description: Site Kpi Summaries Query's page.
    suboptions:
      limit:
        description: Limit.
        type: int
      offset:
        description: Offset.
        type: int
      sortBy:
        description: Site Kpi Summaries Query's sortBy.
        suboptions:
          name:
            description: Name.
            type: str
          order:
            description: Order.
            type: str
        type: dict
    type: dict
  startTime:
    description: Start Time.
    type: int
  views:
    description: Views.
    elements: str
    type: list
requirements:
  - dnacentersdk >= 2.11.0
  - python >= 3.12
seealso:
  - name: Cisco DNA Center documentation for Sites GetSiteAnalyticsForTheChildSitesOfGivenParentSiteAndOtherFilters
    description: Complete reference of the GetSiteAnalyticsForTheChildSitesOfGivenParentSiteAndOtherFilters API.
    link: https://developer.cisco.com/docs/dna-center/#!get-site-analytics-for-the-child-sites-of-given-parent-site-and-other-filters
notes:
  - SDK Method used are
    sites.Sites.get_site_analytics_for_the_child_sites_of_given_parent_site_and_other_filters,
  - Paths used are
    post /dna/data/api/v1/siteKpiSummaries/query,
"""

EXAMPLES = r"""
---
- name: Create
  cisco.dnac.site_kpi_summaries_query:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    attributes:
      - string
    endTime: 0
    filters:
      - key: string
        operator: string
        value: string
    headers: '{{my_headers | from_json}}'
    page:
      limit: 0
      offset: 0
      sortBy:
        name: string
        order: string
    startTime: 0
    views:
      - string
"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "response": {
        "taskLocation": "string",
        "taskId": "string"
      },
      "version": "string"
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: event_series_count_info
short_description: Information module for Event Series Count
description:
  - Get all Event Series Count.
  - Get the Count of Published Notifications.
version_added: '3.1.0'
extends_documentation_fragment:
  - cisco.dnac.module_info
author: Rafael Campos (@racampos)
options:
  headers:
    description: Additional headers.
    type: dict
  eventIds:
    description:
      - EventIds query parameter. The registered EventId should be provided.
    type: str
  startTime:
    description:
      - StartTime query parameter. Start Time in milliseconds.
    type: float
  endTime:
    description:
      - EndTime query parameter. End Time in milliseconds.
    type: float
  category:
    description:
      - Category query parameter.
    type: str
  type:
    description:
      - Type query parameter.
    type: str
  severity:
    description:
      - Severity query parameter.
    type: str
  domain:
    description:
      - Domain query parameter.
    type: str
  subDomain:
    description:
      - SubDomain query parameter. Sub Domain.
    type: str
  source:
    description:
      - Source query parameter.
    type: str
requirements:
  - dnacentersdk >= 2.11.0
  - python >= 3.12
seealso:
  - name: Cisco DNA Center documentation for Event Management CountOfNotifications
    description: Complete reference of the CountOfNotifications API.
    link: https://developer.cisco.com/docs/dna-center/#!count-of-notifications
notes:
  - SDK Method used are
    event_management.EventManagement.count_of_notifications,
  - Paths used are
    get /dna/intent/api/v1/event/event-series/count,
"""

EXAMPLES = r"""
---
- name: Get all Event Series Count
  cisco.dnac.event_series_count_info:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    headers: "{{my_headers | from_json}}"
    eventIds: string
    startTime: 0
    endTime: 0
    category: string
    type: string
    severity: string
    domain: string
    subDomain: string
    source: string
  register: result
"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: str
  sample: >
    "string"
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: interface_network_device_detail_info
short_description: Information module for Interface Network Device Detail
description:
  - Get all Interface Network Device Detail.
  - Returns interface by specified device Id and interface name.
version_added: '3.1.0'
extends_documentation_fragment:
  - cisco.dnac.module_info
author: Rafael Campos (@racampos)
options:
  headers:
    description: Additional headers.
    type: dict
  deviceId:
    description:
      - DeviceId path parameter. Device ID.
    type: str
  name:
    description:
      - Name query parameter. Interface name.
    type: str
requirements:
  - dnacentersdk >= 2.11.0
  - python >= 3.12
seealso:
  - name: Cisco DNA Center documentation for Devices GetInterfaceDetailsByDeviceIdAndInterfaceName
    description: Complete reference of the GetInterfaceDetailsByDeviceIdAndInterfaceName API.
    link: https://developer.cisco.com/docs/dna-center/#!get-interface-details-by-device-id-and-interface-name
notes:
  - SDK Method used are
    devices.Devices.get_interface_details,
  - Paths used are
    get /dna/intent/api/v1/interface/network-device/{deviceId}/interface-name,
"""

EXAMPLES = r"""
---
- name: Get all Interface Network Device Detail
  cisco.dnac.interface_network_device_detail_info:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    headers: "{{my_headers | from_json}}"
    name: string
    deviceId: string
  register: result
"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "response": {
        "addresses": [
          {
            "address": {
              "ipAddress": {
                "address": "string"
              },
              "ipMask": {
                "address": "string"
              },
              "isInverseMask": true
            },
            "type": "string"
          }
        ],
        "adminStatus": "string",
        "className": "string",
        "description": "string",
        "name": "string",
        "deviceId": "string",
        "duplex": "string",
        "id": "string",
        "ifIndex": "string",
        "instanceTenantId": "string",
        "instanceUuid": "string",
        "interfaceType": "string",
        "ipv4Address": "string",
        "ipv4Mask": "string",
        "isisSupport": "string",
        "lastOutgoingPacketTime": 0,
        "lastIncomingPacketTime": 0,
        "lastUpdated": "string",
        "macAddress": "string",
        "mappedPhysicalInterfaceId": "string",
        "mappedPhysicalInterfaceName": "string",
        "mediaType": "string",
        "mtu": "string",
        "nativeVlanId": "string",
        "ospfSupport": "string",
        "pid": "string",
        "portMode": "string",
        "portName": "string",
        "portType": "string",
        "serialNo": "string",
        "series": "string",
        "speed": "string",
        "status": "string",
        "vlanId": "string",
        "voiceVlan": "string",
        "poweroverethernet": "string",
        "networkdevice_id": "string",
        "managedComputeElement": "string",
        "managedNetworkElement": "string",
        "managedNetworkElementUrl": "string",
        "managedComputeElementUrl": "string"
      },
      "version": "string"
    }
"""



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright (c) 2025, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type

from ansible.module_utils.common import validation


def validate_str(item, param_spec, param_name, invalid_params, module=None):
    """
    This function checks that the input `item` is a valid string and conforms to
    the constraints specified in `param_spec`. If the string is not valid or does
    not meet the constraints, an error message is added to `invalid_params`.

    Args:
        item (str): The input string to be validated.
        param_spec (dict): The parameter's specification, including validation constraints.
        param_name (str): The name of the parameter being validated.
        invalid_params (list): A list to collect validation error messages.
        module (object, optional): Ansible module object, required if any parameter has `no_log` enabled.

    Returns:
        str: The validated and possibly normalized string.

    Example `param_spec`:
        {
            "type": "str",
            "length_max": 255 # Optional: maximum allowed length
        }
    """

    try:
        item = validation.check_type_str(item, False)
    except TypeError as e:
        invalid_params.append(
            f"'{param_name}': '{item}' is invalid. Reason: {str(e)}. "
        )
        return item

    max_length = param_spec.get("length_max")
    if max_length:
        if 1 <= len(item) <= max_length:
            return item
        else:
            invalid_params.append(
                "{0}:{1} : The string exceeds the allowed "
                "range of max {2} char".format(
                    param_name, item, param_spec.get("length_max")
                )
            )

    return item


def validate_float(item, param_spec, param_name, invalid_params, module=None):
    """
    This function checks that the input `item` is a valid float and conforms to
    the constraints specified in `param_spec`. If the float is not valid or does
    not meet the constraints, an error message is added to `invalid_params`.

    Args:
        item (float): The input float to be validated.
        param_spec (dict): The parameter's specification, including validation constraints.
        param_name (str): The name of the parameter being validated.
        invalid_params (list): A list to collect validation error messages.
        module (object, optional): Ansible module object, required if any parameter has `no_log` enabled.

    Returns:
        float: The validated float.
    """
    try:
        item = validation.check_type_float(item)
    except TypeError as e:
        invalid_params.append(
            f"'{param_name}': '{item}' is invalid. Reason: {str(e)}. "
        )

    return item


def validate_integer_within_range(
    item, param_spec, param_name, invalid_params, module=None
):
    """
    This function checks that the input `item` is a valid integer and conforms to
    the constraints specified in `param_spec`. If the integer is not valid or does
    not meet the constraints, an error message is added to `invalid_params`.

    Args:
        item (int): The input integer to be validated.
        param_spec (dict): The parameter's specification, including validation constraints.
        param_name (str): The name of the parameter being validated.
        invalid_params (list): A list to collect validation error messages.
        module (object, optional): Ansible module object, required if any parameter has `no_log` enabled.

    Returns:
        int: The validated integer.

    Example `param_spec`:
        {
            "type": "int",
            "range_min": 1,     # Optional: minimum allowed value
            "range_max": 100    # Optional: maximum allowed value
        }
    """
    try:
        item = validation.check_type_int(item)
    except TypeError as e:
        invalid_params.append(
            f"'{param_name}': '{item}' is invalid. Reason: {str(e)}. "
        )
        return item

    min_value = param_spec.get("range_min", 1)
    if param_spec.get("range_max") and not (
        min_value <= item <= param_spec["range_max"]
    ):
        invalid_params.append(
            "{0}: {1} : The item exceeds the allowed range of min: {2} and max: {3}".format(
                param_name,
                item,
                param_spec.get("range_min"),
                param_spec.get("range_max"),
            )
        )

    return item


def validate_bool(item, param_spec, param_name, invalid_params, module=None):
    """
    This function checks that the input `item` is a valid boolean value. If it does
    not represent a valid boolean value, an error message is added to `invalid_params`.

    Args:
        item (bool): The input boolean value to be validated.
        param_spec (dict): The parameter's specification, including validation constraints.
        param_name (str): The name of the parameter being validated.
        invalid_params (list): A list to collect validation error messages.
        module (object, optional): Ansible module object, required if any parameter has `no_log` enabled.

    Returns:
        bool: The validated boolean value.
    """
    try:
        item = validation.check_type_bool(item)
    except TypeError as e:
        invalid_params.append(
            f"'{param_name}': '{item}' is invalid. Reason: {str(e)}. "
        )

    return item


def validate_list(item, param_spec, param_name, invalid_params, module=None):
    """
    This function checks if the input `item` is a valid list based on the specified `param_spec`.
    It also verifies that the elements of the list match the expected data type specified in the
    `param_spec`. If any validation errors occur, they are appended to the `invalid_params` list.

    Args:
        item (list): The input list to be validated.
        param_spec (dict): The parameter's specification, including validation constraints.
        param_name (str): The name of the parameter being validated.
        invalid_params (list): A list to collect validation error messages.
        module (object, optional): Ansible module object, required if any parameter has `no_log` enabled.

    Returns:
        list: The validated list, potentially normalized based on the specification.
    """

    try:
        if param_spec.get("type") == type(item).__name__:
            keys_list = []
            for dict_key in param_spec:
                keys_list.append(dict_key)
            if len(keys_list) == 1:
                return validation.check_type_list(item)

            temp_dict = {keys_list[1]: param_spec[keys_list[1]]}
            try:
                if param_spec["elements"]:
                    if param_spec["elements"] == "dict":
                        common_defaults = {
                            "type",
                            "elements",
                            "required",
                            "default",
                            "choices",
                            "no_log",
                        }
                        filtered_param_spec = {
                            key: value
                            for key, value in param_spec.items()
                            if key not in common_defaults
                        }
                        if filtered_param_spec:
                            item, list_invalid_params = validate_list_of_dicts(
                                item, filtered_param_spec
                            )
                            invalid_params.extend(list_invalid_params)

                    get_spec_type = param_spec["type"]
                    get_spec_element = param_spec["elements"]
                    if type(item).__name__ == get_spec_type:
                        for element in item:
                            if type(element).__name__ != get_spec_element:
                                invalid_params.append(
                                    "{0} is not of the same datatype as expected which is {1}".format(
                                        element, get_spec_element
                                    )
                                )
                    else:
                        invalid_params.append(
                            "{0} is not of the same datatype as expected which is {1}".format(
                                item, get_spec_type
                            )
                        )
            except Exception as e:
                item, list_invalid_params = validate_list_of_dicts(item, temp_dict)
                invalid_params.extend(list_invalid_params)
        else:
            invalid_params.append(
                f"'{param_name}': '{item}' is invalid. Reason: expected type: '{param_spec.get('type')}'. "
                f"Provided type: '{type(item).__name__}'. "
            )
    except Exception as e:
        invalid_params.append("{0} : comes into the exception".format(e))

    return item


def validate_dict(item, param_spec, param_name, invalid_params, module=None):
    """
    This function checks if the input `item` is a valid dictionary based on the specified `param_spec`.
    If the dictionary does not match the expected data type specified in the `param_spec`,
    a validation error is appended to the `invalid_params` list.

    Args:
        item (dict): The input dictionary to be validated.
        param_spec (dict): The parameter's specification, including validation constraints.
        param_name (str): The name of the parameter being validated.
        invalid_params (list): A list to collect validation error messages.
        module (object, optional): Ansible module object, required if any parameter has `no_log` enabled.

    Returns:
        dict: The validated dictionary.
    """
    if param_spec.get("type") != type(item).__name__:
        invalid_params.append(
            f"'{param_name}': '{item}' is invalid. Reason: expected type: '{param_spec.get('type')}'. "
            f"Provided type: '{type(item).__name__}'. "
        )
        return item

    if param_spec.get("type") == "dict":
        common_defaults = {
            "type",
            "elements",
            "required",
            "default",
            "choices",
            "no_log",
        }
        filtered_param_spec = {
            key: value
            for key, value in param_spec.items()
            if key not in common_defaults
        }

        valid_params_dict = {}

        if filtered_param_spec:
            for param in filtered_param_spec:
                curr_item = item.get(param)
                if curr_item is None:
                    if filtered_param_spec[param].get("required"):
                        invalid_params.append(
                            "{0} : Required parameter not found".format(param)
                        )
                    else:
                        curr_item = filtered_param_spec[param].get("default")
                        valid_params_dict[param] = curr_item
                    continue
                data_type = filtered_param_spec[param].get("type")
                switch = {
                    "str": validate_str,
                    "int": validate_integer_within_range,
                    "float": validate_float,
                    "bool": validate_bool,
                    "list": validate_list,
                    "dict": validate_dict,
                    "raw": lambda item, *_: item,
                }

                validator = switch.get(data_type)
                if validator:
                    curr_item = validator(
                        curr_item,
                        filtered_param_spec[param],
                        param,
                        invalid_params,
                        module,
                    )
                else:
                    invalid_params.append(
                        "{0}:{1} : Unsupported data type {2}.".format(
                            param, curr_item, data_type
                        )
                    )

                choice = filtered_param_spec[param].get("choices")
                if choice:
                    if curr_item not in choice:
                        invalid_params.append(
                            "{0} : Invalid choice provided".format(curr_item)
                        )

                no_log = filtered_param_spec[param].get("no_log")
                if no_log:
                    if module is not None:
                        module.no_log_values.add(curr_item)
                    else:
                        msg = "\n\n'{0}' is a no_log parameter".format(param)
                        msg += "\nAnsible module object must be passed to this "
                        msg += "\nfunction to ensure it is not logged\n\n"
                        raise Exception(msg)

                valid_params_dict[param] = curr_item
            item = valid_params_dict

    return validation.check_type_dict(item)


def validate_list_of_dicts(param_list, spec, module=None):
    """Validate/Normalize playbook params. Will raise when invalid parameters found.
    param_list: a playbook parameter list of dicts
    spec: an argument spec dict
        e.g. spec = dict(ip=dict(required=True, type='bool'),
                        foo=dict(type='str', default='bar'))
    return: list of normalized input data
    """

    v = validation
    normalized = []
    invalid_params = []
    for list_entry in param_list:
        valid_params_dict = {}
        if not spec:
            # Handle the case when spec becomes empty but param list is still there
            invalid_params.append("No more spec to validate, but parameters remain")
            break
        for param in spec:
            item = list_entry.get(param)
            if item is None:
                if spec[param].get("required"):
                    invalid_params.append(
                        "{0} : Required parameter not found".format(param)
                    )
                else:
                    item = spec[param].get("default")
                    valid_params_dict[param] = item
                continue
            data_type = spec[param].get("type")
            switch = {
                "str": validate_str,
                "int": validate_integer_within_range,
                "float": validate_float,
                "bool": validate_bool,
                "list": validate_list,
                "dict": validate_dict,
                "raw": lambda item, *_: item,
            }

            validator = switch.get(data_type)
            if validator:
                item = validator(item, spec[param], param, invalid_params, module)
            else:
                invalid_params.append(
                    "{0}:{1} : Unsupported data type {2}.".format(
                        param, item, data_type
                    )
                )

            choice = spec[param].get("choices")
            if choice:
                if item not in choice:
                    invalid_params.append("{0} : Invalid choice provided".format(item))

            no_log = spec[param].get("no_log")
            if no_log:
                if module is not None:
                    module.no_log_values.add(item)
                else:
                    msg = "\n\n'{0}' is a no_log parameter".format(param)
                    msg += "\nAnsible module object must be passed to this "
                    msg += "\nfunction to ensure it is not logged\n\n"
                    raise Exception(msg)

            valid_params_dict[param] = item
        normalized.append(valid_params_dict)

    return normalized, invalid_params



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: backup_configuration
short_description: Resource module for Backup Configuration
description:
  - Manage operation create of the resource Backup Configuration.
  - This api is used to create or update backup configuration.
version_added: '6.18.0'
extends_documentation_fragment:
  - cisco.dnac.module
author: Rafael Campos (@racampos)
options:
  dataRetention:
    description: Date retention policy of the backup.
    type: int
  encryptionPassphrase:
    description: Password to encrypt the backup information.
    type: str
  mountPath:
    description: Backup storage mount path.
    type: str
  type:
    description: The storage type.
    type: str
requirements:
  - dnacentersdk >= 2.11.0
  - python >= 3.12
seealso:
  - name: Cisco DNA Center documentation for Backup CreateBackupConfiguration
    description: Complete reference of the CreateBackupConfiguration API.
    link: https://developer.cisco.com/docs/dna-center/#!create-backup-configuration
notes:
  - SDK Method used are
    backup.Backup.create_backup_configuration,
  - Paths used are
    post /dna/system/api/v1/backupConfiguration,
"""

EXAMPLES = r"""
---
- name: Create
  cisco.dnac.backup_configuration:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    state: present
    dataRetention: 0
    encryptionPassphrase: string
    mountPath: string
    type: string
"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "response": {
        "message": "string"
      },
      "version": "string"
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: wireless_profiles_id_policy_tags_bulk
short_description: Resource module for Wireless Profiles Id Policy Tags Bulk
description:
  - Manage operation create of the resource Wireless Profiles Id Policy Tags Bulk. - > This endpoint allows the creation of
    multiple `Policy Tags` associated with a specific `Wireless Profile` in a single request. The `id` of the Wireless Profile
    must be provided as a path parameter, and a list of `Policy Tags` should be included in the request body.
version_added: '6.17.0'
extends_documentation_fragment:
  - cisco.dnac.module
author: Rafael Campos (@racampos)
options:
  id:
    description: Id path parameter. Wireless Profile Id.
    type: str
  items:
    description: Wireless Profiles Id Policy Tags Bulk's items.
    elements: dict
    suboptions:
      apZones:
        description: Ap Zones.
        elements: str
        type: list
      policyTagName:
        description: Use English letters, numbers, special characters except <, /, '.*', ? and leading/trailing space.
        type: str
      siteIds:
        description: Site Ids.
        elements: str
        type: list
    type: list
requirements:
  - dnacentersdk >= 2.11.0
  - python >= 3.12
seealso:
  - name: Cisco DNA Center documentation for Wireless CreateMultiplePolicyTagsForAWirelessProfileInBulk
    description: Complete reference of the CreateMultiplePolicyTagsForAWirelessProfileInBulk API.
    link: https://developer.cisco.com/docs/dna-center/#!create-multiple-policy-tags-for-a-wireless-profile-in-bulk
notes:
  - SDK Method used are
    wireless.Wireless.create_multiple_policy_tags_for_a_wireless_profile_in_bulk,
  - Paths used are
    post /dna/intent/api/v1/wirelessProfiles/{id}/policyTags/bulk,
"""

EXAMPLES = r"""
---
- name: Create
  cisco.dnac.wireless_profiles_id_policy_tags_bulk:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    id: string
    items:
      - apZones:
          - string
        policyTagName: string
        siteIds:
          - string
"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "response": {
        "taskId": "string",
        "url": "string"
      },
      "version": "string"
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: sda_virtual_network_info
short_description: Information module for Sda Virtual Network
description:
  - Get all Sda Virtual Network.
  - Get virtual network VN from SDA Fabric.
version_added: '3.1.0'
extends_documentation_fragment:
  - cisco.dnac.module_info
author: Rafael Campos (@racampos)
options:
  headers:
    description: Additional headers.
    type: dict
  virtualNetworkName:
    description:
      - VirtualNetworkName query parameter.
    type: str
  siteNameHierarchy:
    description:
      - SiteNameHierarchy query parameter.
    type: str
requirements:
  - dnacentersdk >= 2.11.0
  - python >= 3.12
seealso:
  - name: Cisco DNA Center documentation for SDA GetVNFromSDAFabric
    description: Complete reference of the GetVNFromSDAFabric API.
    link: https://developer.cisco.com/docs/dna-center/#!get-vn-from-sda-fabric
notes:
  - SDK Method used are
    sda.Sda.get_vn,
  - Paths used are
    get /dna/intent/api/v1/business/sda/virtual-network,
"""

EXAMPLES = r"""
---
- name: Get all Sda Virtual Network
  cisco.dnac.sda_virtual_network_info:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    headers: "{{my_headers | from_json}}"
    virtualNetworkName: string
    siteNameHierarchy: string
  register: result
"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "siteNameHierarchy": "string",
      "virtualNetworkName": "string",
      "fabricName": "string",
      "isInfraVN": true,
      "isDefaultVN": true,
      "virtualNetworkContextId": "string",
      "virtualNetworkId": "string",
      "status": "string",
      "description": "string",
      "executionId": "string"
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: site_health_summaries_trend_analytics_info
short_description: Information module for Site Health Summaries Trend Analytics
description:
  - Get all Site Health Summaries Trend Analytics. - > Retrieves the time series information of health and issue data for
    sites specified by query parameters, or all sites. The data will be grouped based on the specified trend time interval.
version_added: '6.17.0'
extends_documentation_fragment:
  - cisco.dnac.module_info
author: Rafael Campos (@racampos)
options:
  headers:
    description: Additional headers.
    type: dict
  startTime:
    description:
      - >
        StartTime query parameter. Start time from which API queries the data set related to the resource. It
        must be specified in UNIX epochtime in milliseconds. Value is inclusive.
    type: float
  endTime:
    description:
      - >
        EndTime query parameter. End time to which API queries the data set related to the resource. It must be
        specified in UNIX epochtime in milliseconds. Value is inclusive.
    type: float
  siteHierarchy:
    description:
      - >
        SiteHierarchy query parameter. The full hierarchical breakdown of the site tree starting from Global
        site name and ending with the specific site name. The Root site is named "Global" (Ex.
        `Global/AreaName/BuildingName/FloorName`) This field supports wildcard asterisk (`*`) character search
        support. E.g. `*/San*, */San, /San*` Examples `?siteHierarchy=Global/AreaName/BuildingName/FloorName`
        (single siteHierarchy requested) `?siteHierarchy=Global/AreaName/BuildingName/FloorName&siteHierarchy=Gl
        obal/AreaName2/BuildingName2/FloorName2` (multiple siteHierarchies requested).
    type: str
  siteHierarchyId:
    description:
      - >
        SiteHierarchyId query parameter. The full hierarchy breakdown of the site tree in id form starting from
        Global site UUID and ending with the specific site UUID. (Ex.
        `globalUuid/areaUuid/buildingUuid/floorUuid`) This field supports wildcard asterisk (`*`) character
        search support. E.g. `*uuid*, *uuid, uuid*` Examples
        `?siteHierarchyId=globalUuid/areaUuid/buildingUuid/floorUuid `(single siteHierarchyId requested) `?siteH
        ierarchyId=globalUuid/areaUuid/buildingUuid/floorUuid&siteHierarchyId=globalUuid/areaUuid2/buildingUuid2
        /floorUuid2` (multiple siteHierarchyIds requested).
    type: str
  siteType:
    description:
      - >
        SiteType query parameter. The type of the site. A site can be an area, building, or floor. Default when
        not provided will be `floor,building,area` Examples `?siteType=area` (single siteType requested)
        `?siteType=area&siteType=building&siteType=floor` (multiple siteTypes requested).
    type: str
  id:
    description:
      - >
        Id query parameter. The list of entity Uuids. (Ex."6bef213c-19ca-4170-8375-b694e251101c") Examples
        id=6bef213c-19ca-4170-8375-b694e251101c (single entity uuid requested) id=6bef213c-19ca-4170-8375-
        b694e251101c&id=32219612-819e-4b5e-a96b-cf22aca13dd9&id=2541e9a7-b80d-4955-8aa2-79b233318ba0 (multiple
        entity uuid with '&' separator).
    type: str
  trendInterval:
    description:
      - >
        TrendInterval query parameter. The time window to aggregate the metrics. Interval can be 5 minutes or 10
        minutes or 1 hour or 1 day or 7 days.
    type: str
  limit:
    description:
      - Limit query parameter. Maximum number of records to return.
    type: int
  offset:
    description:
      - >
        Offset query parameter. Specifies the starting point within all records returned by the API. It's one
        based offset. The starting value is 1.
    type: int
  timeSortOrder:
    description:
      - TimeSortOrder query parameter. The sort order of a time sorted API response.
    type: str
  attribute:
    description:
      - >
        Attribute query parameter. Supported Analytics Attributes networkDeviceCount,
        networkDeviceGoodHealthCount,wirelessDeviceCount, wirelessDeviceGoodHealthCount, accessDeviceCount,
        accessDeviceGoodHealthCount, coreDeviceCount, coreDeviceGoodHealthCount, distributionDeviceCount,
        distributionDeviceGoodHealthCount, routerDeviceCount, routerDeviceGoodHealthCount, apDeviceCount,
        apDeviceGoodHealthCount, wlcDeviceCount, wlcDeviceGoodHealthCount, switchDeviceCount,
        switchDeviceGoodHealthCount, networkDeviceGoodHealthPercentage, accessDeviceGoodHealthPercentage,
        coreDeviceGoodHealthPercentage, distributionDeviceGoodHealthPercentage,
        routerDeviceGoodHealthPercentage, apDeviceGoodHealthPercentage, wlcDeviceGoodHealthPercentage,
        switchDeviceGoodHealthPercentage, wirelessDeviceGoodHealthPercentage, clientCount,
        clientGoodHealthCount, wiredClientCount, wirelessClientCount, wiredClientGoodHealthCount,
        wirelessClientGoodHealthCount, clientGoodHealthPercentage, wiredClientGoodHealthPercentage,
        wirelessClientGoodHealthPercentage, clientDataUsage, p1IssueCount, p2IssueCount, p3IssueCount,
        p4IssueCount, issueCount attribute=networkDeviceCount (single attribute requested)
        attribute=networkDeviceCount&attribute=clientCount (multiple attributes requested).
    type: str
  taskId:
    description:
      - >
        TaskId query parameter. Used to retrieve asynchronously processed & stored data. When this parameter is
        used, the rest of the request params will be ignored.
    type: str
requirements:
  - dnacentersdk >= 2.11.0
  - python >= 3.12
seealso:
  - name: Cisco DNA Center documentation for Sites ReadTrendAnalyticsDataForAGroupingOfSitesInYourNetwork
    description: Complete reference of the ReadTrendAnalyticsDataForAGroupingOfSitesInYourNetwork API.
    link: https://developer.cisco.com/docs/dna-center/#!read-trend-analytics-data-for-a-grouping-of-sites-in-your-network
notes:
  - SDK Method used are
    sites.Sites.read_trend_analytics_data_for_a_grouping_of_sites_in_your_network,
  - Paths used are
    get /dna/data/api/v1/siteHealthSummaries/trendAnalytics,
"""

EXAMPLES = r"""
---
- name: Get all Site Health Summaries Trend Analytics
  cisco.dnac.site_health_summaries_trend_analytics_info:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    headers: "{{my_headers | from_json}}"
    startTime: 0
    endTime: 0
    siteHierarchy: string
    siteHierarchyId: string
    siteType: string
    id: string
    trendInterval: string
    limit: 0
    offset: 0
    timeSortOrder: string
    attribute: string
    taskId: string
  register: result
"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "response": [
        {
          "timestamp": 0,
          "attributes": [
            {
              "name": "string",
              "value": 0
            }
          ]
        }
      ],
      "page": {
        "limit": 0,
        "offset": 0,
        "count": 0,
        "timeSortOrder": "string"
      },
      "version": "string"
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: assurance_tasks_info
short_description: Information module for Assurance Tasks
description:
  - Get all Assurance Tasks.
  - Get Assurance Tasks by id.
  - returns a task given a specific task id.
  - returns all existing tasks in a paginated list.
version_added: '6.17.0'
extends_documentation_fragment:
  - cisco.dnac.module_info
author: Rafael Campos (@racampos)
options:
  headers:
    description: Additional headers.
    type: dict
  limit:
    description:
      - Limit query parameter. Maximum number of records to return.
    type: int
  offset:
    description:
      - >
        Offset query parameter. Specifies the starting point within all records returned by the API. It's one
        based offset. The starting value is 1.
    type: int
  sortBy:
    description:
      - SortBy query parameter. A field within the response to sort by.
    type: str
  order:
    description:
      - Order query parameter. The sort order of the field ascending or descending.
    type: str
  status:
    description:
      - Status query parameter. Used to get a subset of tasks by their status.
    type: str
  id:
    description:
      - Id path parameter. Unique task id.
    type: str
requirements:
  - dnacentersdk >= 2.11.0
  - python >= 3.12
seealso:
  - name: Cisco DNA Center documentation for Task RetrieveAListOfAssuranceTasks
    description: Complete reference of the RetrieveAListOfAssuranceTasks API.
    link: https://developer.cisco.com/docs/dna-center/#!retrieve-a-list-of-assurance-tasks
  - name: Cisco DNA Center documentation for Task RetrieveASpecificAssuranceTaskById
    description: Complete reference of the RetrieveASpecificAssuranceTaskById API.
    link: https://developer.cisco.com/docs/dna-center/#!retrieve-a-specific-assurance-task-by-id
notes:
  - SDK Method used are
    task.Task.retrieve_a_list_of_assurance_tasks,
    task.Task.retrieve_a_specific_assurance_task_by_id,
  - Paths used are
    get /dna/data/api/v1/assuranceTasks,
    get /dna/data/api/v1/assuranceTasks/{id},
"""

EXAMPLES = r"""
---
- name: Get all Assurance Tasks
  cisco.dnac.assurance_tasks_info:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    headers: "{{my_headers | from_json}}"
    limit: 0
    offset: 0
    sortBy: string
    order: string
    status: string
  register: result
- name: Get Assurance Tasks by id
  cisco.dnac.assurance_tasks_info:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    headers: "{{my_headers | from_json}}"
    id: string
  register: result
"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "response": {
        "id": "string",
        "status": "string",
        "startTime": 0,
        "endTime": 0,
        "updateTime": 0,
        "progress": "string",
        "failureReason": "string",
        "errorCode": "string",
        "requestType": "string",
        "data": {},
        "resultUrl": "string"
      },
      "version": "string"
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: icap_settings_configuration_models_preview_activity_id_deploy
short_description: Resource module for Icap Settings Configuration Models Preview Activity Id Deploy
description:
  - Manage operation create of the resource Icap Settings Configuration Models Preview Activity Id Deploy. - > Deploys the
    ICAP configuration intent by activity ID, which was returned in property "taskId" of the TaskResponse of the POST. POST'ing
    the intent prior to generating the intent CLI for preview-approve has the same effect as direct-deploy'ing the intent
    to the device.
version_added: '6.17.0'
extends_documentation_fragment:
  - cisco.dnac.module
author: Rafael Campos (@racampos)
options:
  previewActivityId:
    description: PreviewActivityId path parameter. Activity from the POST /deviceConfigugrationModels task response.
    type: str
requirements:
  - dnacentersdk >= 2.11.0
  - python >= 3.12
seealso:
  - name: Cisco DNA Center documentation for Sensors DeploysTheICAPConfigurationIntentByActivityID
    description: Complete reference of the DeploysTheICAPConfigurationIntentByActivityID API.
    link: https://developer.cisco.com/docs/dna-center/#!deploys-the-icap-configuration-intent-by-activity-id
notes:
  - SDK Method used are
    sensors.Sensors.deploys_the_i_cap_configuration_intent_by_activity_id,
  - Paths used are
    post /dna/intent/api/v1/icapSettings/configurationModels/{previewActivityId}/deploy,
"""

EXAMPLES = r"""
---
- name: Create
  cisco.dnac.icap_settings_configuration_models_preview_activity_id_deploy:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    previewActivityId: string
"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "response": {
        "taskId": "string",
        "url": "string"
      },
      "version": "string"
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Ansible module to manage CheckPoint Firewall (c) 2019
#
# Ansible is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Ansible is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.
#

from __future__ import (absolute_import, division, print_function)

__metaclass__ = type

ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['preview'],
                    'supported_by': 'community'}

DOCUMENTATION = """
---
module: cp_mgmt_nat_rule
short_description: Manages nat-rule objects on Checkpoint over Web Services API.
description:
  - Manages nat-rule objects on Checkpoint devices including creating, updating and removing objects.
  -  Minimum version required is 1.7.1 and JHF with PMTR-88097.
  - All operations are performed over Web Services API.
  - Available from R80 management version.
version_added: "5.0.0"
author: "Eden Brillant (@chkp-edenbr)"
options:
  package:
    description:
      - Name of the package.
      - Available from R80.10 management version.
    type: str
  position:
    description:
      - Position in the rulebase. The use of values "top" and "bottom" may not be idempotent.
    type: str
  relative_position:
    description:
      - Position in the rulebase.
      - Use of this field may not be idempotent.
    type: dict
    suboptions:
      below:
        description:
          - Add rule below specific rule/section identified by name (limited to 50 rules if
            search_entire_rulebase is False).
        type: str
      above:
        description:
          - Add rule above specific rule/section identified by name (limited to 50 rules if
            search_entire_rulebase is False).
        type: str
      top:
        description:
          - Add rule to the top of a specific section identified by name (limited to 50 rules if
            search_entire_rulebase is False).
        type: str
      bottom:
        description:
          - Add rule to the bottom of a specific section identified by name (limited to 50 rules if
            search_entire_rulebase is False).
        type: str
  search_entire_rulebase:
    description:
      - Whether to search the entire rulebase for a rule that's been edited in its relative_position field to make sure
        there indeed has been a change in its position or the section it might be in.
    type: bool
    default: False
  name:
    description:
      - Rule name.
      - Available from R81 management version.
    type: str
    required: True
  enabled:
    description:
      - Enable/Disable the rule.
    type: bool
  install_on:
    description:
      - Which Gateways identified by the name or UID to install the policy on.
    type: list
    elements: str
  method:
    description:
      - Nat method.
    type: str
    choices: ['static', 'hide', 'nat64', 'nat46', 'cgnat']
  original_destination:
    description:
      - Original destination.
    type: str
  original_service:
    description:
      - Original service.
    type: str
  original_source:
    description:
      - Original source.
    type: str
  translated_destination:
    description:
      - Translated  destination.
    type: str
  translated_service:
    description:
      - Translated  service.
    type: str
  translated_source:
    description:
      - Translated  source.
    type: str
  comments:
    description:
      - Comments string.
    type: str
  details_level:
    description:
      - The level of detail for some of the fields in the response can vary from showing only the UID value of the object to a fully detailed
        representation of the object.
    type: str
    choices: ['uid', 'standard', 'full']
  ignore_warnings:
    description:
      - Apply changes ignoring warnings.
    type: bool
  ignore_errors:
    description:
      - Apply changes ignoring errors. You won't be able to publish such a changes. If ignore-warnings flag was omitted - warnings will also be ignored.
    type: bool
extends_documentation_fragment: check_point.mgmt.checkpoint_objects
"""

EXAMPLES = """
- name: add-nat-rule
  cp_mgmt_nat_rule:
    name: nat_rule1
    comments: comment example1 nat999
    enabled: false
    install_on:
      - Policy Targets
    original_destination: All_Internet
    original_source: Any
    package: standard
    position: 1
    state: present

- name: set-nat-rule
  cp_mgmt_nat_rule:
    name: nat_rule1
    comments: rule for RND members  RNDNetwork-> RND to Internal Network
    enabled: false
    original_service: ssh_version_2
    original_source: Any
    package: standard
    state: present

- name: delete-nat-rule
  cp_mgmt_nat_rule:
    name: nat_rule1
    package: standard
    state: absent
"""

RETURN = """
cp_mgmt_nat_rule:
  description: The checkpoint object created or updated.
  returned: always, except when deleting the object.
  type: dict
"""

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.check_point.mgmt.plugins.module_utils.checkpoint import checkpoint_argument_spec_for_objects, api_call_for_rule


def main():
    argument_spec = dict(
        package=dict(type='str'),
        position=dict(type='str'),
        relative_position=dict(type='dict', options=dict(
            below=dict(type='str'),
            above=dict(type='str'),
            top=dict(type='str'),
            bottom=dict(type='str')
        )),
        search_entire_rulebase=dict(type='bool', default=False),
        name=dict(type='str', required=True),
        enabled=dict(type='bool'),
        install_on=dict(type='list', elements='str'),
        method=dict(type='str', choices=['static', 'hide', 'nat64', 'nat46', 'cgnat']),
        original_destination=dict(type='str'),
        original_service=dict(type='str'),
        original_source=dict(type='str'),
        translated_destination=dict(type='str'),
        translated_service=dict(type='str'),
        translated_source=dict(type='str'),
        comments=dict(type='str'),
        details_level=dict(type='str', choices=['uid', 'standard', 'full']),
        ignore_warnings=dict(type='bool'),
        ignore_errors=dict(type='bool')
    )
    argument_spec.update(checkpoint_argument_spec_for_objects)

    module = AnsibleModule(argument_spec=argument_spec, supports_check_mode=True)
    api_call_object = 'nat-rule'

    if module.params["relative_position"] is not None:
        if module.params["position"] is not None:
            raise AssertionError("The use of both 'relative_position' and 'position' arguments isn't allowed")
        module.params["position"] = module.params["relative_position"]
    module.params.pop("relative_position")

    result = api_call_for_rule(module, api_call_object)
    module.exit_json(**result)


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: dnacaap_management_execution_status_info
short_description: Information module for Dnacaap Management Execution Status
description:
  - Get Dnacaap Management Execution Status by id.
  - Retrieves the execution details of a Business API.
version_added: '4.0.0'
extends_documentation_fragment:
  - cisco.dnac.module_info
author: Rafael Campos (@racampos)
options:
  headers:
    description: Additional headers.
    type: dict
  executionId:
    description:
      - ExecutionId path parameter. Execution Id of API.
    type: str
requirements:
  - dnacentersdk >= 2.11.0
  - python >= 3.12
seealso:
  - name: Cisco DNA Center documentation for Task GetBusinessAPIExecutionDetails
    description: Complete reference of the GetBusinessAPIExecutionDetails API.
    link: https://developer.cisco.com/docs/dna-center/#!get-business-api-execution-details
notes:
  - SDK Method used are
    task.Task.get_business_api_execution_details,
  - Paths used are
    get /dna/intent/api/v1/dnacaap/management/execution-status/{executionId},
"""

EXAMPLES = r"""
---
- name: Get Dnacaap Management Execution Status by id
  cisco.dnac.dnacaap_management_execution_status_info:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    headers: "{{my_headers | from_json}}"
    executionId: string
  register: result
"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "bapiKey": "string",
      "bapiName": "string",
      "bapiExecutionId": "string",
      "startTime": "string",
      "startTimeEpoch": 0,
      "endTime": "string",
      "endTimeEpoch": 0,
      "timeDuration": 0,
      "status": "string",
      "bapiSyncResponse": "string",
      "bapiSyncResponseJson": {},
      "runtimeInstanceId": "string",
      "bapiError": "string"
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: license_register
short_description: Resource module for License Register
description:
  - Manage operation create of the resource License Register.
  - Registers the system with Cisco Smart Software Manager CSSM .
version_added: '6.17.0'
extends_documentation_fragment:
  - cisco.dnac.module
author: Rafael Campos (@racampos)
options:
  smartAccountId:
    description: The ID of the Smart Account to which the system is registered.
    type: str
  virtualAccountId:
    description: The ID of the Virtual Account to which the system is registered.
    type: str
requirements:
  - dnacentersdk >= 2.11.0
  - python >= 3.12
seealso:
  - name: Cisco DNA Center documentation for Licenses SystemLicensingRegistration
    description: Complete reference of the SystemLicensingRegistration API.
    link: https://developer.cisco.com/docs/dna-center/#!system-licensing-registration
notes:
  - SDK Method used are
    licenses.Licenses.system_licensing_registration,
  - Paths used are
    post /dna/system/api/v1/license/register,
"""

EXAMPLES = r"""
---
- name: Create
  cisco.dnac.license_register:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    smartAccountId: string
    virtualAccountId: string
"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "response": {
        "url": "string"
      },
      "version": "string"
    }
"""
